@@ -20,7 +20,7 @@
  */
 
 #include <fcntl.h>
-#include <unistd.h>
+#include "stdinc.h"
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
@@ -81,11 +81,11 @@ list_load_from_data (GSList ** list, char *ibuf, int size)
 	{
 		if (*buf != '#')
 		{
-			if (!strncasecmp (buf, "NAME ", 5))
+			if (!g_ascii_strncasecmp (buf, "NAME ", 5))
 			{
 				g_strlcpy (name, buf + 5, sizeof (name));
 			}
-			else if (!strncasecmp (buf, "CMD ", 4))
+			else if (!g_ascii_strncasecmp (buf, "CMD ", 4))
 			{
 				g_strlcpy (cmd, buf + 4, sizeof (cmd));
 				if (*name)
@@ -134,10 +134,10 @@ regex_list_load_from_data (GSList **list, char *ibuf, int size)
 	{
 		if (*buf != '#')
 		{
-			if (!strncasecmp(buf, "NAME ", 5))
+			if (!g_ascii_strncasecmp(buf, "NAME ", 5))
 			{
 				g_strlcpy(name, buf + 5, sizeof(name));
-			} else if (!strncasecmp(buf, "CMD ", 4))
+			} else if (!g_ascii_strncasecmp(buf, "CMD ", 4))
 			{
 				g_strlcpy(cmd, buf + 4, sizeof(cmd));
 				if (*name)
@@ -272,7 +272,7 @@ cfg_get_str (char *cfg, const char *var, char **dest)
 {
 	while (1)
 	{
-		if (!strncasecmp (var, cfg, strlen (var)))
+		if (!g_ascii_strncasecmp (var, cfg, strlen (var)))
 		{
 			char *value, t;
 			cfg += strlen (var);
@@ -395,8 +395,8 @@ char *
 get_xdir_fs (void)
 {
 	if (!xdir_fs)
-		xdir_fs = g_strdup_printf ("%s/" XCHAT_DIR, g_get_home_dir ());
-
+		xdir_fs = g_build_filename(g_get_home_dir(), XCHAT_DIR, NULL);
+	
 	return xdir_fs;
 }
 
@@ -541,6 +541,8 @@ PrefsEntry vars[] = {
 	{"input_tray_hilight", PREFS_TYPE_BOOL, &prefs.input_tray_hilight},
 	{"input_tray_priv", PREFS_TYPE_BOOL, &prefs.input_tray_priv},
 
+        {"hilight_enable", PREFS_TYPE_BOOL, &prefs.hilight_enable},
+
 	{"irc_auto_rejoin", PREFS_TYPE_BOOL, &prefs.autorejoin},
 	{"irc_ban_type", PREFS_TYPE_INT, &prefs.bantype},
 	{"irc_conf_mode", PREFS_TYPE_BOOL, &prefs.confmode},
@@ -562,6 +564,7 @@ PrefsEntry vars[] = {
 	{"irc_real_name", PREFS_TYPE_STR, &prefs.realname},
 	{"irc_servernotice", PREFS_TYPE_BOOL, &prefs.servernotice},
 	{"irc_skip_motd", PREFS_TYPE_BOOL, &prefs.skipmotd},
+	{"irc_time_format", PREFS_TYPE_STR, &prefs.irc_time_format},
 	{"irc_user_name", PREFS_TYPE_STR, &prefs.username},
 	{"irc_wallops", PREFS_TYPE_BOOL, &prefs.wallops},
 	{"irc_who_join", PREFS_TYPE_BOOL, &prefs.userhost},
@@ -629,6 +632,9 @@ PrefsEntry vars[] = {
 	{"text_tint_red", PREFS_TYPE_INT, &prefs.tint_red},
 	{"text_transparent", PREFS_TYPE_BOOL, &prefs.transparent},
 	{"text_wordwrap", PREFS_TYPE_BOOL, &prefs.wordwrap},
+	{"text_overflow_start", PREFS_TYPE_STR, &prefs.text_overflow_start},
+	{"text_overflow_stop", PREFS_TYPE_STR, &prefs.text_overflow_stop},
+	{"text_overflow_limit", PREFS_TYPE_INT, &prefs.text_overflow_limit},
 
 	{0, 0, 0},
 };
@@ -720,9 +726,7 @@ load_config (void)
 	prefs.dcctimeout = 180;
 	prefs.dccstalltimeout = 60;
 	prefs.notify_timeout = 15;
-	prefs.tint_red =
-		prefs.tint_green =
-		prefs.tint_blue = 195;
+	prefs.tint_red = prefs.tint_green = prefs.tint_blue = 195;
 	prefs.auto_indent = 1;
 	prefs.max_auto_indent = 256;
 	prefs.show_separator = 1;
@@ -766,6 +770,16 @@ load_config (void)
 	prefs.font_normal = strdup(DEF_FONT);
 	prefs.dnsprogram = strdup("host");
 	prefs.irc_no_hilight = strdup("NickServ,ChanServ");
+	prefs.irc_time_format = strdup("%a %b %e %T %Z %Y");
+
+	prefs.gui_ulist_pos = 2;
+	prefs.tab_pos = 1;
+
+	prefs.hilight_enable = TRUE;
+
+	prefs.text_overflow_start = strdup("");
+	prefs.text_overflow_stop  = strdup("");
+	prefs.text_overflow_limit = 0;
 
 	g_free ((char *)username);
 	g_free ((char *)realname);
@@ -814,45 +828,6 @@ load_config (void)
 	sp = strchr (prefs.username, ' ');
 	if (sp)
 		sp[0] = 0;	/* spaces in username would break the login */
-
-	/* try to make sense of old ulist/tree position settings */
-	if (prefs.gui_ulist_pos == 0)
-	{
-		prefs.gui_ulist_pos = 3;	/* top right */
-		if (prefs._gui_ulist_left)
-			prefs.gui_ulist_pos = 2;	/* bottom left */
-
-		switch (prefs._tabs_position)
-		{
-		case 0:
-			prefs.tab_pos = 6; /* bottom */
-			break;
-		case 1:
-			prefs.tab_pos = 5;	/* top */
-			break;
-		case 2:
-			prefs.tab_pos = 1; 	/* left */
-			break;
-		case 3:
-			prefs.tab_pos = 4; 	/* right */
-			break;
-		case 4:
-			prefs.tab_pos = 1;	/* (hidden)left */
-			break;
-		case 5:
-			if (prefs._gui_ulist_left)
-			{
-				prefs.tab_pos = 1; 	/* above ulist left */
-				prefs.gui_ulist_pos = 2;
-			}
-			else
-			{
-				prefs.tab_pos = 3; 	/* above ulist right */
-				prefs.gui_ulist_pos = 4;
-			}
-			break;
-		}
-	}
 }
 
 int
@@ -992,7 +967,7 @@ cfg_get_bool (char *var)
 	return -1;
 }
 
-int
+CommandResult
 cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	int wild = FALSE;
@@ -1027,7 +1002,7 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (!*var)
 	{
 		set_list (sess, tbuf);
-		return TRUE;
+		return CMD_EXEC_OK;
 	}
 
 	if ((strchr (var, '*') || strchr (var, '?')) && !*val)
@@ -1051,8 +1026,8 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			case PREFS_TYPE_STR:
 				if (erase || *val)
 				{
-					g_free(vars[i].ptr);
-					vars[i].ptr = g_strdup(val);
+					g_free(*((char **) vars[i].ptr));
+					*((char **) vars[i].ptr) = g_strdup(val);
 					if (!quiet)
 						PrintTextf (sess, "%s set to: %s\n", var, *((char **) vars[i].ptr));
 				} else
@@ -1100,22 +1075,27 @@ cmd_set (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	if (!finds && !quiet)
 		PrintText (sess, "No such variable.\n");
 
-	return TRUE;
+	return CMD_EXEC_OK;
 }
 
 int
 xchat_open_file (char *file, int flags, int mode, int xof_flags)
 {
-	char buf[1024];
+	int ret;
+	char *buf;
 
 	if (xof_flags & XOF_FULLPATH)
 		return open (file, flags | OFLAGS);
 
-	snprintf (buf, sizeof (buf), "%s/%s", get_xdir_fs (), file);
+	buf = g_build_filename(get_xdir_fs(), file, NULL);
 	if (xof_flags & XOF_DOMODE)
-		return open (buf, flags | OFLAGS, mode);
+		ret = open (buf, flags | OFLAGS, mode);
 	else
-		return open (buf, flags | OFLAGS);
+		ret = open (buf, flags | OFLAGS);
+
+	g_free(buf);
+
+	return ret;
 }
 
 FILE *
