@@ -18,9 +18,9 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <unistd.h>
+#include "stdinc.h"
 #include <stdlib.h>
-#include <sys/utsname.h>
+
 #include <time.h>
 
 #include "xchat.h"
@@ -28,12 +28,13 @@
 #include "util.h"
 #include "modes.h"
 #include "outbound.h"
-#include "ignore.h"
+#include "ignore-ng.h"
 #include "inbound.h"
 #include "dcc.h"
 #include "text.h"
 #include "ctcp.h"
 #include "server.h"
+#include "userlist.h"
 #include "xchatc.h"
 
 
@@ -83,96 +84,97 @@ ctcp_check (session *sess, char *nick, char *word[], char *word_eol[],
 }
 
 void
-ctcp_handle (session *sess, char *to, char *nick,
-				 char *msg, char *word[], char *word_eol[], int id)
+ctcp_handle (session *sess, char *to, char *nick, char *ip, char *msg, char *word[], char *word_eol[], int id)
 {
 	char *po;
-	session *chansess;
 	server *serv = sess->server;
 	char outbuf[1024];
+	gchar *hostmask;
+	gboolean channel = is_channel(serv, to);
+	IgnoreLevel level = (channel) ? IGNORE_PUBLIC : IGNORE_PRIVATE;
+        level |= IGNORE_CTCP;
+
+	if (*ip != '\0')
+		hostmask = g_strjoin("!", nick, ip, NULL);
+	else
+		hostmask = g_strdup(nick);
 
 	/* consider DCC to be different from other CTCPs */
-	if (!strncasecmp (msg, "DCC", 3))
+	if (!g_ascii_strncasecmp (msg, "DCC", 3))
 	{
 		/* but still let CTCP replies override it */
 		if (!ctcp_check (sess, nick, word, word_eol, word[4] + 2))
 		{
-			if (!ignore_check (word[1], IG_DCC))
+			if (!ignore_check (hostmask, IGNORE_DCC))
 				handle_dcc (sess, nick, word, word_eol);
 		}
+		g_free(hostmask);
 		return;
 	}
 
 	/* consider ACTION to be different from other CTCPs. Check
       ignore as if it was a PRIV/CHAN. */
-	if (!strncasecmp (msg, "ACTION ", 7))
+	if (!g_ascii_strncasecmp (msg, "ACTION ", 7))
 	{
-		if (is_channel (serv, to))
-		{
-			/* treat a channel action as a CHAN */
-			if (ignore_check (word[1], IG_CHAN))
-				return;
-		} else
-		{
-			/* treat a private action as a PRIV */
-			if (ignore_check (word[1], IG_PRIV))
-				return;
-		}
-
 		/* but still let CTCP replies override it */
 		if (ctcp_check (sess, nick, word, word_eol, word[4] + 2))
 			goto generic;
 
 		inbound_action (sess, to, nick, msg + 7, FALSE, id);
+		g_free(hostmask);
 		return;
 	}
 
-	if (ignore_check (word[1], IG_CTCP))
-		return;
+	if (!ignore_check(hostmask, level)) {
+		if (!strcasecmp (msg, "VERSION") && !prefs.hidever)
+		{
+#ifndef _WIN32
+			struct utsname un;
 
-	if (!strcasecmp (msg, "VERSION") && !prefs.hidever)
-	{
-		struct utsname un;
+			uname(&un);
 
-		uname(&un);
+			snprintf (outbuf, sizeof (outbuf), "VERSION conspire "PACKAGE_VERSION" - running on %s %s %s",
+						 un.sysname, un.release, un.machine);
+#else
+			OSVERSIONINFO osvi;
+			SYSTEM_INFO si;
 
-		snprintf (outbuf, sizeof (outbuf), "VERSION conspire "PACKAGE_VERSION" - running on %s %s %s",
-					 un.sysname, un.release, un.machine);
+			osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+			GetVersionEx(&osvi);
+			GetSystemInfo(&si);
 
-		serv->p_nctcp (serv, nick, outbuf);
-	}
+			snprintf(outbuf, sizeof(outbuf), "VERSION conspire "PACKAGE_VERSION" - running on Windows %ld.%ld build %ld (i%d86)",
+				osvi.dwMajorVersion, osvi.dwMinorVersion, osvi.dwBuildNumber,
+				si.wProcessorLevel);
+#endif
 
-	if (!strcasecmp(msg, "CLIENTINFO"))
-	{
-		snprintf(outbuf, sizeof(outbuf), "CLIENTINFO CLIENTINFO PING TIME VERSION");
+			serv->p_nctcp (serv, nick, outbuf);
+		} else if (!strcasecmp(msg, "CLIENTINFO"))
+		{
+			snprintf(outbuf, sizeof(outbuf), "CLIENTINFO CLIENTINFO PING TIME USERINFO VERSION");
 
-		serv->p_nctcp(serv, nick, outbuf);
-	}
+			serv->p_nctcp(serv, nick, outbuf);
+		} else if (!strcasecmp(msg, "USERINFO"))
+		{
+			snprintf(outbuf, sizeof(outbuf), "USERINFO %s", prefs.realname);
 
-	if (!strcasecmp(msg, "TIME")) {
-		time_t time_val = time(NULL);
-		struct tm *tval = localtime(&time_val);
-		char tbuf[200];
-		strftime(tbuf, sizeof(tbuf), prefs.irc_time_format, tval); //Sun Feb  3 18:33:27 CST 2008
-		snprintf(outbuf, sizeof(outbuf), "TIME %s", tbuf);
+			serv->p_nctcp(serv, nick, outbuf);
+		} else if (!strcasecmp(msg, "TIME")) {
+			time_t time_val = time(NULL);
+			struct tm *tval = localtime(&time_val);
+			char tbuf[200];
+			strftime(tbuf, sizeof(tbuf), prefs.irc_time_format, tval); //Sun Feb  3 18:33:27 CST 2008
+			snprintf(outbuf, sizeof(outbuf), "TIME %s", tbuf);
 
-		serv->p_nctcp(serv, nick, outbuf);
-	}
+			serv->p_nctcp(serv, nick, outbuf);
+		}
+        }
 
 generic:
 	po = strchr (msg, '\001');
 	if (po)
 		po[0] = 0;
 
-	if (!is_channel (sess->server, to))
-	{
-		EMIT_SIGNAL (XP_TE_CTCPGEN, sess->server->front_session, msg, nick,
-						 NULL, NULL, 0);
-	} else
-	{
-		chansess = find_channel (sess->server, to);
-		if (!chansess)
-			chansess = sess;
-		EMIT_SIGNAL (XP_TE_CTCPGENC, chansess, msg, nick, to, NULL, 0);
-	}
+	signal_emit("ctcp inbound", 4, sess, msg, hostmask, to);
+	g_free(hostmask);
 }
