@@ -31,7 +31,7 @@
 #include <time.h>
 #include <errno.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#include "stdinc.h"
 #include <fcntl.h>
 
 #define WANTSOCKET
@@ -39,26 +39,20 @@
 #define WANTDNS
 #include "inet.h"
 
+#include "stdinc.h"
+
 #include "xchat.h"
 #include "util.h"
 #include "fe.h"
 #include "outbound.h"
 #include "inbound.h"
 #include "network.h"
-#include "plugin.h"
 #include "server.h"
 #include "text.h"
 #include "url.h"
 #include "xchatc.h"
 #include "base64.h"
-
-#ifdef USE_DCC64
-#define BIG_STR_TO_INT(x) strtoull(x,NULL,10)
-#else
-#define BIG_STR_TO_INT(x) strtoul(x,NULL,10)
-#endif
-
-static char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
+#include "upnp.h"
 
 struct dccstat_info dccstat[] = {
 	{N_("Waiting"), 1 /*black */ },
@@ -69,11 +63,12 @@ struct dccstat_info dccstat[] = {
 	{N_("Aborted"), 4 /*red */ },
 };
 
+const char *dcctypes[] = { "SEND", "RECV", "CHAT", "CHAT" };
+
 static int dcc_global_throttle;	/* 0x1 = sends, 0x2 = gets */
 /*static*/ int dcc_sendcpssum, dcc_getcpssum;
 
 static struct DCC *new_dcc (void);
-static void dcc_close (struct DCC *dcc, int dccstat, int destroy);
 static gboolean dcc_send_data (GIOChannel *, GIOCondition, struct DCC *);
 static gboolean dcc_read (GIOChannel *, GIOCondition, struct DCC *);
 static gboolean dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc);
@@ -246,10 +241,7 @@ dcc_check_timeouts (void)
 					if (!dcc->throttled
 						&& tim - dcc->lasttime > prefs.dccstalltimeout)
 					{
-						EMIT_SIGNAL (XP_TE_DCCSTALL, dcc->serv->front_session,
-										 dcctypes[dcc->type],
-										 file_part (dcc->file), dcc->nick, NULL, 0);
-						dcc_close (dcc, STAT_ABORTED, FALSE);
+						signal_emit("dcc stoned", 1, dcc);
 					}
 				}
 			}
@@ -261,10 +253,7 @@ dcc_check_timeouts (void)
 				{
 					if (prefs.dcctimeout > 0)
 					{
-						EMIT_SIGNAL (XP_TE_DCCTOUT, dcc->serv->front_session,
-										 dcctypes[dcc->type],
-										 file_part (dcc->file), dcc->nick, NULL, 0);
-						dcc_close (dcc, STAT_ABORTED, FALSE);
+						signal_emit("dcc stoned", 1, dcc);
 					}
 				}
 			}
@@ -346,9 +335,12 @@ dcc_connect_sok (struct DCC *dcc)
 	return sok;
 }
 
-static void
+void
 dcc_close (struct DCC *dcc, int dccstat, int destroy)
 {
+	if (dcc->port)
+		upnp_rem_redir(dcc->port);
+
 	if (dcc->wiotag)
 	{
 		g_source_remove (dcc->wiotag);
@@ -378,7 +370,7 @@ dcc_close (struct DCC *dcc, int dccstat, int destroy)
 		{
 			/* if we just completed a dcc recieve, move the */
 			/* completed file to the completed directory */
-			if(dcc->type == TYPE_RECV)
+			if(dcc->type == TYPE_RECV && prefs.dcc_completed_dir != NULL)
 			{			
 				/* mgl: change this to use destfile_fs for correctness and to */
 				/* handle the case where dccwithnick is set */
@@ -427,21 +419,7 @@ dcc_abort (session *sess, struct DCC *dcc)
 		case STAT_CONNECTING:
 		case STAT_ACTIVE:
 			dcc_close (dcc, STAT_ABORTED, FALSE);
-			switch (dcc->type)
-			{
-			case TYPE_CHATSEND:
-			case TYPE_CHATRECV:
-				EMIT_SIGNAL (XP_TE_DCCCHATABORT, sess, dcc->nick, NULL, NULL,
-								 NULL, 0);
-				break;
-			case TYPE_SEND:
-				EMIT_SIGNAL (XP_TE_DCCSENDABORT, sess, dcc->nick,
-								 file_part (dcc->file), NULL, NULL, 0);
-				break;
-			case TYPE_RECV:
-				EMIT_SIGNAL (XP_TE_DCCRECVABORT, sess, dcc->nick,
-								 dcc->file, NULL, NULL, 0);
-			}
+			signal_emit("dcc abort", 1, dcc);
 			break;
 		default:
 			dcc_close (dcc, 0, TRUE);
@@ -478,8 +456,7 @@ dcc_write_chat (char *nick, char *text)
 	if (dcc && dcc->dccstat == STAT_ACTIVE)
 	{
 		len = strlen (text);
-		tcp_send_real (NULL, dcc->sok, dcc->serv->encoding, dcc->serv->using_irc,
-							text, len);
+		send (dcc->sok, text, len, 0);
 		send (dcc->sok, "\n", 1, 0);
 		dcc->size += len;
 		fe_dcc_update (dcc);
@@ -495,14 +472,11 @@ static int
 dcc_chat_line (struct DCC *dcc, char *line)
 {
 	session *sess;
-	char *word[PDIWORDS];
 	char *po;
 	char *utf;
 	char *conv;
-	int ret, i;
 	int len;
 	gsize utf_len;
-	char portbuf[32];
 
 	len = strlen (line);
 	if (dcc->serv->using_cp1255)
@@ -531,18 +505,6 @@ dcc_chat_line (struct DCC *dcc, char *line)
 	if (!sess)
 		sess = dcc->serv->front_session;
 
-	sprintf (portbuf, "%d", dcc->port);
-
-	word[0] = "DCC Chat Text";
-	word[1] = net_ip (dcc->addr);
-	word[2] = portbuf;
-	word[3] = dcc->nick;
-	word[4] = line;
-	for (i = 5; i < PDIWORDS; i++)
-		word[i] = "\000";
-
-	ret = plugin_emit_print (sess, word);
-
 	/* did the plugin close it? */
 	if (!g_slist_find (dcc_list, dcc))
 	{
@@ -553,19 +515,9 @@ dcc_chat_line (struct DCC *dcc, char *line)
 		return 1;
 	}
 
-	/* did the plugin eat the event? */
-	if (ret)
-	{
-		if (utf)
-			g_free (utf);
-		if (conv)
-			g_free (conv);
-		return 0;
-	}
-
 	url_check_line (line, len);
 
-	if (line[0] == 1 && !strncasecmp (line + 1, "ACTION", 6))
+	if (line[0] == 1 && !g_ascii_strncasecmp (line + 1, "ACTION", 6))
 	{
 		po = strchr (line + 8, '\001');
 		if (po)
@@ -610,10 +562,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 					return TRUE;
 			}
 			sprintf (portbuf, "%d", dcc->port);
-			EMIT_SIGNAL (XP_TE_DCCCHATF, dcc->serv->front_session, dcc->nick,
-							 net_ip (dcc->addr), portbuf,
-							 errorstring ((len < 0) ? sock_error () : 0), 0);
-			dcc_close (dcc, STAT_FAILED, FALSE);
+			signal_emit("dcc chat failed", 3, dcc, portbuf, errorstring((len < 0) ? sock_error() : 0));
 			return TRUE;
 		}
 		i = 0;
@@ -645,7 +594,7 @@ dcc_read_chat (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	}
 }
 
-static void
+void
 dcc_calc_average_cps (struct DCC *dcc)
 {
 	time_t sec;
@@ -704,8 +653,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				old = dcc->destfile;
 				dcc->destfile = xchat_filename_to_utf8 (buf, -1, 0, 0, 0);
 
-				EMIT_SIGNAL (XP_TE_DCCRENAME, dcc->serv->front_session,
-								 old, dcc->destfile, NULL, NULL, 0);
+				signal_emit("dcc file rename", 2, dcc, old);
 				g_free (old);
 			}
 			dcc->fp =
@@ -716,9 +664,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	if (dcc->fp == -1)
 	{
 		/* the last executed function is open(), errno should be valid */
-		EMIT_SIGNAL (XP_TE_DCCFILEERR, dcc->serv->front_session, dcc->destfile,
-						 errorstring (errno), NULL, NULL, 0);
-		dcc_close (dcc, STAT_FAILED, FALSE);
+		signal_emit("dcc file error", 2, dcc, errorstring(errno));
 		return TRUE;
 	}
 	while (1)
@@ -748,23 +694,15 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 					return TRUE;
 				}
 			}
-			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
-							 dcc->destfile, dcc->nick,
-							 errorstring ((n < 0) ? sock_error () : 0), 0);
-			/* send ack here? but the socket is dead */
-			/*if (need_ack)
-				dcc_send_ack (dcc);*/
-			dcc_close (dcc, STAT_FAILED, FALSE);
+			signal_emit("dcc recv error", 2, dcc, errorstring( (n < 0) ? sock_error() : 0));
 			return TRUE;
 		}
 
 		if (write (dcc->fp, buf, n) == -1) /* could be out of hdd space */
 		{
-			EMIT_SIGNAL (XP_TE_DCCRECVERR, dcc->serv->front_session, dcc->file,
-							 dcc->destfile, dcc->nick, errorstring (errno), 0);
 			if (need_ack)
 				dcc_send_ack (dcc);
-			dcc_close (dcc, STAT_FAILED, FALSE);
+			signal_emit("dcc recv error", 2, dcc, errorstring(errno));
 			return TRUE;
 		}
 
@@ -775,11 +713,7 @@ dcc_read (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		if (dcc->pos >= dcc->size)
 		{
 			dcc_send_ack (dcc);
-			dcc_close (dcc, STAT_DONE, FALSE);
-			dcc_calc_average_cps (dcc);	/* this must be done _after_ dcc_close, or dcc_remove_from_sum will see the wrong value in dcc->cps */
-			sprintf (buf, "%d", dcc->cps);
-			EMIT_SIGNAL (XP_TE_DCCRECVCOMP, dcc->serv->front_session,
-							 dcc->file, dcc->destfile, dcc->nick, buf, 0);
+			signal_emit("dcc file complete", 1, dcc);
 			return TRUE;
 		}
 	}
@@ -809,9 +743,7 @@ dcc_did_connect (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		er = sock_error ();
 		if (er != EISCONN)
 		{
-			EMIT_SIGNAL (XP_TE_DCCCONFAIL, dcc->serv->front_session,
-							 dcctypes[dcc->type], dcc->nick, errorstring (er),
-							 NULL, 0);
+			signal_emit("dcc failed", 2, dcc, errorstring(er));
 			dcc->dccstat = STAT_FAILED;
 			fe_dcc_update (dcc);
 			return FALSE;
@@ -842,8 +774,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	{
 	case TYPE_RECV:
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read, dcc);
-		EMIT_SIGNAL (XP_TE_DCCCONRECV, dcc->serv->front_session,
-						 dcc->nick, host, dcc->file, NULL, 0);
+		signal_emit("dcc connected", 2, dcc, host);
 		break;
 	case TYPE_SEND:
 		/* passive send */
@@ -852,8 +783,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 			dcc->wiotag = fe_input_add (dcc->sok, FIA_WRITE, dcc_send_data, dcc);
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_ack, dcc);
 		dcc_send_data (NULL, 0, (gpointer)dcc);
-		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
-						 dcc->nick, host, dcc->file, NULL, 0);
+		signal_emit("dcc connected", 2, dcc, host);
 		break;
 	case TYPE_CHATSEND:	/* pchat */
 		dcc_open_query (dcc->serv, dcc->nick);
@@ -861,8 +791,7 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
-		EMIT_SIGNAL (XP_TE_DCCCONCHAT, dcc->serv->front_session,
-						 dcc->nick, host, NULL, NULL, 0);
+		signal_emit("dcc connected", 2, dcc, host);
 		break;
 	}
 	dcc->starttime = time (0);
@@ -1449,10 +1378,7 @@ dcc_send_data (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	{
 abortit:
 		free (buf);
-		EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick,
-						 errorstring (sock_error ()), NULL, 0);
-		dcc_close (dcc, STAT_FAILED, FALSE);
+		signal_emit("dcc send failed", 2, dcc, errorstring(sock_error()));
 		return TRUE;
 	}
 	if (sent > 0)
@@ -1481,7 +1407,6 @@ static gboolean
 dcc_handle_new_ack (struct DCC *dcc)
 {
 	guint32 ack;
-	char buf[16];
 	gboolean done = FALSE;
 
 	memcpy (&ack, dcc->ack_buf, 4);
@@ -1500,12 +1425,7 @@ dcc_handle_new_ack (struct DCC *dcc)
 	/* DCC complete check */
 	if (dcc->pos >= dcc->size && dcc->ack >= (dcc->size & 0xffffffff))
 	{
-		dcc->ack = dcc->size;	/* force 100% ack for >4 GB */
-		dcc_close (dcc, STAT_DONE, FALSE);
-		dcc_calc_average_cps (dcc);	/* this must be done _after_ dcc_close, or dcc_remove_from_sum will see the wrong value in dcc->cps */
-		sprintf (buf, "%d", dcc->cps);
-		EMIT_SIGNAL (XP_TE_DCCSENDCOMP, dcc->serv->front_session,
-						 file_part (dcc->file), dcc->nick, buf, NULL, 0);
+		signal_emit("dcc send complete", 1, dcc);
 		done = TRUE;
 	}
 	else if ((!dcc->fastsend) && (dcc->ack >= (dcc->pos & 0xffffffff)))
@@ -1513,12 +1433,10 @@ dcc_handle_new_ack (struct DCC *dcc)
 		dcc_send_data (NULL, 0, (gpointer)dcc);
 	}
 
-#ifdef USE_DCC64
 	/* take the top 32 of "bytes send" and bottom 32 of "ack" */
 	dcc->ack = (dcc->pos & G_GINT64_CONSTANT (0xffffffff00000000)) |
 					(dcc->ack & 0xffffffff);
 	/* dcc->ack is only used for CPS and PERCENTAGE calcs from now on... */
-#endif
 
 	return done;
 }
@@ -1527,6 +1445,7 @@ static gboolean
 dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 {
 	int len;
+	gchar *error;
 
 	while (1)
 	{
@@ -1539,10 +1458,8 @@ dcc_read_ack (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 				if (would_block ())	/* ok - keep waiting */
 					return TRUE;
 			}
-			EMIT_SIGNAL (XP_TE_DCCSENDFAIL, dcc->serv->front_session,
-							 file_part (dcc->file), dcc->nick,
-							 errorstring ((len < 0) ? sock_error () : 0), NULL, 0);
-			dcc_close (dcc, STAT_FAILED, FALSE);
+			error = errorstring((len < 0) ? sock_error() : 0);
+			signal_emit("dcc send failed", 2, dcc, error);
 			return TRUE;
 		}
 
@@ -1596,8 +1513,7 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 			dcc->wiotag = fe_input_add (sok, FIA_WRITE, dcc_send_data, dcc);
 		dcc->iotag = fe_input_add (sok, FIA_READ|FIA_EX, dcc_read_ack, dcc);
 		dcc_send_data (NULL, 0, (gpointer)dcc);
-		EMIT_SIGNAL (XP_TE_DCCCONSEND, dcc->serv->front_session,
-						 dcc->nick, host, dcc->file, NULL, 0);
+		signal_emit("dcc connected", 2, dcc, host);
 		break;
 
 	case TYPE_CHATSEND:
@@ -1605,8 +1521,7 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 		dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_read_chat, dcc);
 		dcc->dccchat = malloc (sizeof (struct dcc_chat));
 		dcc->dccchat->pos = 0;
-		EMIT_SIGNAL (XP_TE_DCCCONCHAT, dcc->serv->front_session,
-						 dcc->nick, host, NULL, NULL, 0);
+		signal_emit("dcc connected", 2, dcc, host);
 		break;
 	}
 
@@ -1722,6 +1637,8 @@ dcc_listen_init (struct DCC *dcc, session *sess)
 
 	dcc->iotag = fe_input_add (dcc->sok, FIA_READ|FIA_EX, dcc_accept, dcc);
 
+	upnp_add_redir (inet_ntoa(SAddr.sin_addr), dcc->port);
+
 	return TRUE;
 }
 
@@ -1789,14 +1706,6 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 	if (stat (file_fs, &st) != -1)
 	{
 
-#ifndef USE_DCC64
-		if (sizeof (st.st_size) > 4 && st.st_size > 4294967295U)
-		{
-			PrintText (sess, "Cannot send files larger than 4 GB.\n");
-			goto xit;
-		}
-#endif
-
 		if (!(*file_part (file_fs)) || S_ISDIR (st.st_mode) || st.st_size < 1)
 		{
 			PrintText (sess, "Cannot send directories or empty files.\n");
@@ -1853,8 +1762,7 @@ dcc_send (struct session *sess, char *to, char *file, int maxcps, int passive)
 				}
 				sess->server->p_ctcp (sess->server, to, outbuf);
 
-				EMIT_SIGNAL (XP_TE_DCCOFFER, sess, file_part (dcc->file),
-								 to, dcc->file, NULL, 0);
+				signal_emit("dcc send request", 3, sess, dcc, to);
 			} else
 			{
 				dcc_close (dcc, 0, TRUE);
@@ -2101,7 +2009,7 @@ dcc_get_nick (struct session *sess, char *nick)
 		list = list->next;
 	}
 	if (sess)
-		EMIT_SIGNAL (XP_TE_DCCIVAL, sess, NULL, NULL, NULL, NULL, 0);
+		signal_emit("dcc invalid", 1, sess);
 }
 
 static struct DCC *
@@ -2131,7 +2039,7 @@ dcc_chat (struct session *sess, char *nick, int passive)
 		case STAT_ACTIVE:
 		case STAT_QUEUED:
 		case STAT_CONNECTING:
-			EMIT_SIGNAL (XP_TE_DCCCHATREOFFER, sess, nick, NULL, NULL, NULL, 0);
+			signal_emit("dcc chat duplicate", 2, sess, nick);
 			return;
 		case STAT_ABORTED:
 		case STAT_FAILED:
@@ -2182,7 +2090,7 @@ dcc_chat (struct session *sess, char *nick, int passive)
 						 dcc->addr, dcc->port);
 		}
 		dcc->serv->p_ctcp (dcc->serv, nick, outbuf);
-		EMIT_SIGNAL (XP_TE_DCCCHATOFFERING, sess, nick, NULL, NULL, NULL, 0);
+		signal_emit("dcc chat offer", 2, sess, nick);
 	} else
 	{
 		dcc_close (dcc, 0, TRUE);
@@ -2192,7 +2100,7 @@ dcc_chat (struct session *sess, char *nick, int passive)
 static void
 dcc_malformed (struct session *sess, char *nick, char *data)
 {
-	EMIT_SIGNAL (XP_TE_MALFORMED, sess, nick, data, NULL, NULL, 0);
+	signal_emit("dcc malformed", 3, sess, nick, data);
 }
 
 int
@@ -2264,9 +2172,7 @@ dcc_add_chat (session *sess, char *nick, int port, guint32 addr, int pasvid)
 		dcc->nick = strdup (nick);
 		dcc->starttime = time (0);
 
-		EMIT_SIGNAL (XP_TE_DCCCHATOFFER, sess->server->front_session, nick,
-						 NULL, NULL, NULL, 0);
-
+		signal_emit("dcc chat request", 2, sess, nick);
 		if (prefs.autoopendccchatwindow)
 		{
 			if (fe_dcc_open_chat_win (TRUE))	/* already open? add only */
@@ -2348,8 +2254,7 @@ dcc_add_file (session *sess, char *file, DCC_SIZE size, int port, char *nick, gu
 	}
 	sprintf (tbuf, "%"DCC_SFMT, size);
 	snprintf (tbuf + 24, 300, "%s:%d", net_ip (addr), port);
-	EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
-					 file, tbuf, tbuf + 24, 0);
+	signal_emit("dcc file request", 4, sess, nick, file, tbuf);
 
 	return dcc;
 }
@@ -2452,8 +2357,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 				dcc->serv->p_ctcp (dcc->serv, dcc->nick, tbuf);
 			}
 			sprintf (tbuf, "%"DCC_SFMT, dcc->pos);
-			EMIT_SIGNAL (XP_TE_DCCRESUMEREQUEST, sess, nick,
-							 file_part (dcc->file), tbuf, NULL, 0);
+			signal_emit("dcc file resume", 4, sess, nick, dcc, tbuf);
 		}
 		return;
 	}
@@ -2523,8 +2427,7 @@ handle_dcc (struct session *sess, char *nick, char *word[],
 
 	} else
 	{
-		EMIT_SIGNAL (XP_TE_DCCGENERICOFFER, sess->server->front_session,
-						 word_eol[4] + 2, nick, NULL, NULL, 0);
+		signal_emit("dcc generic offer", 3, sess, nick, word_eol[4]+2);
 	}
 }
 
@@ -2535,7 +2438,7 @@ dcc_show_list (struct session *sess)
 	struct DCC *dcc;
 	GSList *list = dcc_list;
 
-	EMIT_SIGNAL (XP_TE_DCCHEAD, sess, NULL, NULL, NULL, NULL, 0);
+	signal_emit("dcc list start", 1, sess);
 	while (list)
 	{
 		dcc = (struct DCC *) list->data;
