@@ -25,7 +25,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <sys/types.h>
-#include <unistd.h>
+#include "stdinc.h"
 #include <time.h>
 
 #define WANTARPA
@@ -34,7 +34,6 @@
 
 #include "xchat.h"
 #include "util.h"
-#include "ignore.h"
 #include "fe.h"
 #include "modes.h"
 #include "notify.h"
@@ -44,9 +43,8 @@
 #include "servlist.h"
 #include "text.h"
 #include "ctcp.h"
-#include "plugin.h"
 #include "xchatc.h"
-
+#include "ignore-ng.h"
 
 void
 clear_channel (session *sess)
@@ -127,8 +125,7 @@ inbound_open_dialog (server *serv, char *from)
 	session *sess;
 
 	sess = new_ircwindow (serv, from, SESS_DIALOG, 0);
-	/* for playing sounds */
-	EMIT_SIGNAL (XP_TE_OPENDIALOG, sess, NULL, NULL, NULL, NULL, 0);
+	signal_emit("query open", 1, sess);
 
 	return sess;
 }
@@ -205,7 +202,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 		if (prefs.input_beep_priv || (sess && sess->beep))
 			sound_beep (sess);
 
-		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, idtext, NULL, 0);
+		signal_emit("message private", 4, sess, from, text, idtext);
 		return;
 	}
 
@@ -215,10 +212,7 @@ inbound_privmsg (server *serv, char *from, char *ip, char *text, int id)
 	if (prefs.input_flash_priv)
 		fe_flash_window (sess);
 
-	if (sess->type == SESS_DIALOG)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
-	else
-		EMIT_SIGNAL (XP_TE_PRIVMSG, sess, from, text, idtext, NULL, 0);
+	signal_emit("message private", 4, sess, from, text, idtext);
 }
 
 static int
@@ -295,7 +289,10 @@ is_hilight (char *from, char *text, session *sess, server *serv)
 		 FromNick (from, prefs.irc_nick_hilight))
 	{
 		if (sess != current_tab)
+		{
 			sess->nick_said = TRUE;
+			lastact_update(sess);
+		}
 		fe_set_hilight (sess);
 		return 1;
 	}
@@ -344,6 +341,7 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 			sess->msg_said = TRUE;
 			sess->new_data = FALSE;
 		}
+		lastact_update(sess);
 	}
 
 	user = userlist_find (sess, from);
@@ -356,6 +354,10 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 	if (!fromme)
 	{
 		hilight = is_hilight (from, text, sess, serv);
+
+		if (!prefs.hilight_enable)
+			hilight = FALSE;
+
 		if (hilight && prefs.input_beep_hilight)
 			beep = TRUE;
 
@@ -367,26 +369,29 @@ inbound_action (session *sess, char *chan, char *from, char *text, int fromme, i
 
 		/* private action, flash? */
 		if (!is_channel (serv, chan) && prefs.input_flash_priv)
-			fe_flash_window (sess);
-
-		if (hilight)
 		{
-			EMIT_SIGNAL (XP_TE_HCHANACTION, sess, from, text, nickchar, NULL, 0);
+			fe_flash_window (sess);
+			signal_emit("action private hilight", 4, sess, from, text, nickchar);
+			return;
+                } else if (hilight)
+		{
+			signal_emit("action public hilight", 4, sess, from, text, nickchar);
 			return;
 		}
 	}
 
 	if (fromme)
-		EMIT_SIGNAL (XP_TE_UACTION, sess, from, text, nickchar, NULL, 0);
+		signal_emit("user action", 4, sess, from, text, nickchar);
+	else if (is_channel(serv, chan))
+		signal_emit("action public", 4, sess, from, text, nickchar);
 	else
-		EMIT_SIGNAL (XP_TE_CHANACTION, sess, from, text, nickchar, NULL, 0);
+		signal_emit("action private", 4, sess, from, text, nickchar);
 }
 
 void
 inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text, char fromme, int id)
 {
 	struct User *user;
-	int hilight = FALSE;
 	char nickchar[2] = "\000";
 	char idtext[64];
 
@@ -409,6 +414,7 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 	{
 		sess->msg_said = TRUE;
 		sess->new_data = FALSE;
+		lastact_update(sess);
 	}
 
 	user = userlist_find (sess, from);
@@ -422,7 +428,7 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 	{
   		if (prefs.auto_unmark_away && serv->is_away)
 			sess->server->p_set_back (sess->server);
-		EMIT_SIGNAL (XP_TE_UCHANMSG, sess, from, text, nickchar, NULL, 0);
+		signal_emit("user message public", 4, sess, from, text, nickchar);
 		return;
 	}
 
@@ -437,24 +443,26 @@ inbound_chanmsg (server *serv, session *sess, char *chan, char *from, char *text
 			fe_tray_set_icon (FE_ICON_MESSAGE);
 	}
 
-	if (is_hilight (from, text, sess, serv))
-	{
-		hilight = TRUE;
-		if (prefs.input_beep_hilight)
-			sound_beep (sess);
-	}
-	else
-	{
-		if (sess->type != SESS_DIALOG && prefs.input_flash_chans)
-			fe_flash_window (sess);
-	}
+	if (prefs.hilight_enable)
+        {
+		if (is_hilight (from, text, sess, serv))
+		{
+			if (prefs.input_beep_hilight)
+				sound_beep (sess);
+			signal_emit("message public hilight", 5, sess, from, text, nickchar, idtext);
+        	        return;
+		}
+		else
+		{
+			if (sess->type != SESS_DIALOG && prefs.input_flash_chans)
+				fe_flash_window (sess);
+		}
+        }
 
 	if (sess->type == SESS_DIALOG)
-		EMIT_SIGNAL (XP_TE_DPRIVMSG, sess, from, text, idtext, NULL, 0);
-	else if (hilight)
-		EMIT_SIGNAL (XP_TE_HCHANMSG, sess, from, text, nickchar, idtext, 0);
+		signal_emit("message private", 4, sess, from, text, idtext);
 	else
-		EMIT_SIGNAL (XP_TE_CHANMSG, sess, from, text, nickchar, idtext, 0);
+		signal_emit("message public", 5, sess, from, text, nickchar, idtext);
 }
 
 void
@@ -480,11 +488,9 @@ inbound_newnick (server *serv, char *nick, char *newnick, int quiet)
 				if (!quiet)
 				{
 					if (me)
-						EMIT_SIGNAL (XP_TE_UCHANGENICK, sess, nick, newnick, NULL,
-										 NULL, 0);
+						signal_emit("user nick changed", 3, sess, nick, newnick);
 					else
-						EMIT_SIGNAL (XP_TE_CHANGENICK, sess, nick, newnick, NULL,
-										 NULL, 0);
+						signal_emit("nick changed", 3, sess, nick, newnick);
 				}
 			}
 			if (sess->type == SESS_DIALOG && !serv->p_cmp (sess->channel, nick))
@@ -581,7 +587,7 @@ inbound_ujoin (server *serv, char *chan, char *nick, char *ip)
 	/* sends a MODE */
 	serv->p_join_info (sess->server, chan);
 
-	EMIT_SIGNAL (XP_TE_UJOIN, sess, nick, chan, ip, NULL, 0);
+	signal_emit("user joined", 4, sess, nick, chan, ip);
 
 	if (prefs.userhost)
 	{
@@ -597,7 +603,7 @@ inbound_ukick (server *serv, char *chan, char *kicker, char *reason)
 	session *sess = find_channel (serv, chan);
 	if (sess)
 	{
-		EMIT_SIGNAL (XP_TE_UKICK, sess, serv->nick, chan, kicker, reason, 0);
+		signal_emit("user kicked", 5, sess, serv->nick, chan, kicker, reason);
 		clear_channel (sess);
 		if (prefs.autorejoin)
 		{
@@ -613,11 +619,7 @@ inbound_upart (server *serv, char *chan, char *ip, char *reason)
 	session *sess = find_channel (serv, chan);
 	if (sess)
 	{
-		if (*reason)
-			EMIT_SIGNAL (XP_TE_UPARTREASON, sess, serv->nick, ip, chan, reason,
-							 0);
-		else
-			EMIT_SIGNAL (XP_TE_UPART, sess, serv->nick, ip, chan, NULL, 0);
+		signal_emit("user part", 5, sess, serv->nick, ip, chan, reason);
 		clear_channel (sess);
 	}
 }
@@ -632,17 +634,17 @@ inbound_nameslist (server *serv, char *chan, char *names)
 	sess = find_channel (serv, chan);
 	if (!sess)
 	{
-		EMIT_SIGNAL (XP_TE_USERSONCHAN, serv->server_session, chan, names, NULL,
-						 NULL, 0);
+		signal_emit("channel users", 3, serv->server_session, chan, names);
 		return;
 	}
 	if (!sess->ignore_names)
-		EMIT_SIGNAL (XP_TE_USERSONCHAN, sess, chan, names, NULL, NULL, 0);
+		signal_emit("channel users", 3, sess, chan, names);
 
 	if (sess->end_of_names)
 	{
 		sess->end_of_names = FALSE;
 		userlist_clear (sess);
+		fe_userlist_numbers_block(sess);
 	}
 
 	while (1)
@@ -682,7 +684,7 @@ inbound_topic (server *serv, char *chan, char *topic_text)
 	} else
 		sess = serv->server_session;
 
-	EMIT_SIGNAL (XP_TE_TOPIC, sess, chan, topic_text, NULL, NULL, 0);
+	signal_emit("channel topic", 3, sess, chan, topic_text);
 }
 
 void
@@ -697,7 +699,7 @@ inbound_topicnew (server *serv, char *nick, char *chan, char *topic)
 		new_topic = strip_color (topic, -1, STRIP_ALL);
 		set_topic (sess, new_topic);
 		free (new_topic);
-		EMIT_SIGNAL (XP_TE_NEWTOPIC, sess, nick, topic, chan, NULL, 0);
+		signal_emit("channel topic changed", 4, sess, nick, topic, chan);
 	}
 }
 
@@ -708,7 +710,7 @@ inbound_join (server *serv, char *chan, char *user, char *ip)
 	if (sess)
 	{
 		if (!sess->hide_join_part)
-			EMIT_SIGNAL (XP_TE_JOIN, sess, user, chan, ip, NULL, 0);
+			signal_emit("channel join", 4, sess, user, chan, ip);
 		userlist_add (sess, user, ip);
 	}
 }
@@ -719,7 +721,7 @@ inbound_kick (server *serv, char *chan, char *user, char *kicker, char *reason)
 	session *sess = find_channel (serv, chan);
 	if (sess)
 	{
-		EMIT_SIGNAL (XP_TE_KICK, sess, kicker, user, chan, reason, 0);
+		signal_emit("channel kick", 5, sess, kicker, user, chan, reason);
 		userlist_remove (sess, user);
 	}
 }
@@ -732,10 +734,7 @@ inbound_part (server *serv, char *chan, char *user, char *ip, char *reason)
 	{
 		if (!sess->hide_join_part)
 		{
-			if (*reason)
-				EMIT_SIGNAL (XP_TE_PARTREASON, sess, user, ip, chan, reason, 0);
-			else
-				EMIT_SIGNAL (XP_TE_PART, sess, user, ip, chan, NULL, 0);
+			signal_emit("channel part", 5, sess, user, ip, chan, reason);
 		}
 		userlist_remove (sess, user);
 	}
@@ -751,7 +750,7 @@ inbound_topictime (server *serv, char *chan, char *nick, time_t stamp)
 		sess = serv->server_session;
 
 	tim[24] = 0;	/* get rid of the \n */
-	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
+	signal_emit("channel topic date", 4, sess, chan, nick, tim);
 }
 
 static gboolean
@@ -759,31 +758,45 @@ netsplit_display_victims(server *serv)
 {
 	GString *buffer;
 	GSList *head;
+	GSList *list = sess_list;
+	session *sess;
 
-	buffer = g_string_new("");
-
-	for (head = serv->split_list; head != NULL; head = head->next)
+	for (; list != NULL; list = list->next)
 	{
-		if (buffer->len < 420)
+		sess = (session *) list->data;
+		if (sess->server == serv && sess->type == SESS_CHANNEL)
 		{
-			g_string_append_printf(buffer, "%s%s",
-				*buffer->str != '\0' ? ", " : "",
-				(gchar *) head->data);
+			buffer = g_string_new("");
+
+			/* tab was affected by netsplit, attempt to display victims for this tab */
+			for (head = sess->split_list; head != NULL; head = head->next)
+			{
+				if (buffer->len < 420)
+				{
+					g_string_append_printf(buffer, "%s%s",
+						*buffer->str != '\0' ? ", " : "",
+						(gchar *) head->data);
+				}
+				else
+				{
+					signal_emit("server netsplit", 3, sess, serv, buffer->str);
+					g_string_erase(buffer, 0, -1);
+				}
+				g_free(head->data);
+			}
+
+			if (buffer->len)
+				signal_emit("server netsplit", 3, sess, serv, buffer->str);
+
+			/* free the list for this window */
+			g_slist_free(sess->split_list);
+			sess->split_list = NULL;
+
+			/* and clear buffer for next one */
+			g_string_free(buffer, TRUE);
 		}
-		else
-		{
-			EMIT_SIGNAL(XP_TE_NS_START, serv->front_session, serv->split_serv1, serv->split_serv2, buffer->str, NULL, 0);
-			g_string_erase(buffer, 0, -1);
-		}
-		g_free(head->data);
 	}
 
-	if (buffer->len)
-		EMIT_SIGNAL(XP_TE_NS_START, serv->front_session, serv->split_serv1, serv->split_serv2, buffer->str, NULL, 0);
-
-	g_string_free(buffer, TRUE);
-	g_slist_free(serv->split_list);
-	serv->split_list = NULL;
 	g_free(serv->split_reason);
 	g_free(serv->split_serv1);
 	g_free(serv->split_serv2);
@@ -794,7 +807,7 @@ netsplit_display_victims(server *serv)
 	return FALSE;
 }
 
-/* check if quit message is a netsplit message -- shamelessly 
+/* check if quit message is a netsplit message -- shamelessly
    stolen from irssi. --nenolod */
 static gboolean
 quitmsg_is_split(const char *msg)
@@ -887,10 +900,10 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 	if (serv->split_reason && !strcmp(serv->split_reason, reason))
 		netsplit = TRUE;
 	else if ((netsplit = quitmsg_is_split(reason)) == TRUE)
-	{		
+	{
 		if (netsplit)
 		{
-			if (serv->split_list)
+			if (serv->split_reason)
 			{
 				if (serv->split_timer)
 					g_source_remove(serv->split_timer);
@@ -913,21 +926,13 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
 		}
 	}
 
-	if (netsplit)
-	{
-		serv->split_list = g_slist_append(serv->split_list, g_strdup(nick));
-
-		if (serv->split_timer)
-			g_source_remove(serv->split_timer);
-
-		serv->split_timer = g_timeout_add(500, (GSourceFunc) netsplit_display_victims, serv);
-	}
-	else if (prefs.strip_quits)
+	if (!netsplit && prefs.strip_quits)
 	{
 		/* strip IRCd Quit: prefix */
 		char *tmp;
 
-		if ((tmp = strstr(reason, "Quit:")) != NULL)
+		if (((tmp = strstr(reason, "Quit:")) != NULL) ||
+		    ((tmp = strstr(reason, "Exit:")) != NULL))
 			reason = tmp + 6; /* skip past Quit: */
 	}
 
@@ -940,12 +945,26 @@ inbound_quit (server *serv, char *nick, char *ip, char *reason)
  				was_on_front_session = TRUE;
 			if (userlist_remove (sess, nick))
 			{
-				if (!netsplit && !sess->hide_join_part)
-					EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
+				if (netsplit)
+				{
+					sess->split_list = g_slist_append(sess->split_list, g_strdup(nick));
+
+					if (serv->split_timer)
+						g_source_remove(serv->split_timer);
+
+					serv->split_timer = g_timeout_add(500, (GSourceFunc) netsplit_display_victims, serv);
+				}
+				else if (!sess->hide_join_part)
+				{
+					signal_emit("channel quit", 4, sess, nick, reason, ip);
+				}
 			}
-			else if (!netsplit && sess->type == SESS_DIALOG &&
+			else if (sess->type == SESS_DIALOG &&
 				!serv->p_cmp (sess->channel, nick))
-				EMIT_SIGNAL (XP_TE_QUIT, sess, nick, reason, ip, NULL, 0);
+			{
+				/* previously, this wasn't displayed for dialog sessions, I think it's a good idea. :) */
+				signal_emit("query quit", 4, sess, nick, reason, ip);
+			}
 		}
 	}
 
@@ -984,11 +1003,11 @@ inbound_ping_reply (session *sess, char *timestring, char *from)
 		if (sess->server->lag_sent)
 			sess->server->lag_sent = 0;
 		else
-			EMIT_SIGNAL (XP_TE_PINGREP, sess, from, "?", NULL, NULL, 0);
+			signal_emit("server ping reply", 3, sess, from, "?");
 	} else
 	{
 		snprintf (outbuf, sizeof (outbuf), "%ld.%ld%ld", dif / 1000000, (dif / 100000) % 10, dif % 10);
-		EMIT_SIGNAL (XP_TE_PINGREP, sess, from, outbuf, NULL, NULL, 0);
+		signal_emit("server ping reply", 3, sess, from, outbuf);
 	}
 }
 
@@ -1014,7 +1033,9 @@ inbound_ctcp_reply (struct session *sess, char *msg, char *nick)
 	char *type = strtok(temp, " ");
 	short len = strlen(type);
 	msg[strlen(msg)-1] = 0;
-	EMIT_SIGNAL (XP_TE_CTCP_REPLY, sess, nick, type, msg+(len+1), NULL, 0);
+
+	signal_emit("ctcp reply", 4, sess, nick, type, msg+(len+1));
+
 	g_free(temp);
 }
 
@@ -1095,7 +1116,7 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 		}
 		if (!sess)
 		{
-			if (server_notice)	
+			if (server_notice)
 				sess = serv->server_session;
 			else
 				sess = serv->front_session;
@@ -1119,11 +1140,11 @@ inbound_notice (server *serv, char *to, char *nick, char *msg, char *ip, int id)
 		po[0] = 0;
 
 	if (server_notice)
-		EMIT_SIGNAL (XP_TE_SERVNOTICE, sess, msg, nick, NULL, NULL, 0);
+		signal_emit("server notice", 3, sess, msg, nick);
 	else if (ptr)
-		EMIT_SIGNAL (XP_TE_CHANNOTICE, sess, nick, to, msg, NULL, 0);
+		signal_emit("notice public", 4, sess, nick, to, msg);
 	else
-		EMIT_SIGNAL (XP_TE_NOTICE, sess, nick, msg, NULL, NULL, 0);
+		signal_emit("notice private", 3, sess, nick, msg);
 }
 
 void
@@ -1144,7 +1165,7 @@ inbound_away (server *serv, char *nick, char *msg)
 
 	sess = serv->front_session;
 
-	EMIT_SIGNAL (XP_TE_WHOIS_AWAY, sess, nick, msg, NULL, NULL, 0);
+	signal_emit("whois away", 3, sess, nick, msg);
 
 	list = sess_list;
 	while (list)
@@ -1172,6 +1193,9 @@ inbound_nameslist_end (server *serv, char *chan)
 			{
 				sess->end_of_names = TRUE;
 				sess->ignore_names = FALSE;
+
+				fe_userlist_numbers_unblock(sess);
+				fe_userlist_numbers(sess);
 			}
 			list = list->next;
 		}
@@ -1243,34 +1267,20 @@ inbound_next_nick (session *sess, char *nick)
 		if (net && !(net->flags & FLAG_USE_GLOBAL) && net->nick2)
 			newnick = g_strdup(net->nick2);
 		serv->p_change_nick (serv, newnick);
-		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, newnick, NULL, NULL, 0);
+		signal_emit("nick clash", 3, sess, nick, newnick);
 		break;
 
 	case 3:
 		newnick = g_strdup(prefs.nick3);
 		serv->p_change_nick (serv, prefs.nick3);
-		EMIT_SIGNAL (XP_TE_NICKCLASH, sess, nick, newnick, NULL, NULL, 0);
+		signal_emit("nick clash", 3, sess, nick, newnick);
 		break;
 
 	default:
-		EMIT_SIGNAL (XP_TE_NICKFAIL, sess, NULL, NULL, NULL, NULL, 0);
+		signal_emit("nick error", 1, sess);
 	}
 }
 
-void
-do_dns (session *sess, char *nick, char *host)
-{
-	char *po;
-	char tbuf[1024];
-
-	po = strrchr (host, '@');
-	if (po)
-		host = po + 1;
-	EMIT_SIGNAL (XP_TE_RESOLVINGUSER, sess, nick, host, NULL, NULL, 0);
-	snprintf (tbuf, sizeof (tbuf), "exec -d %s %s", prefs.dnsprogram, host);
-	handle_command (sess, tbuf, FALSE);
-}
-
 static void
 set_default_modes (server *serv)
 {
@@ -1350,12 +1360,9 @@ inbound_foundip (session *sess, char *ip)
 
 	HostAddr = gethostbyname (ip);
 	if (HostAddr)
-	{
 		prefs.dcc_ip = ((struct in_addr *) HostAddr->h_addr)->s_addr;
-		EMIT_SIGNAL (XP_TE_FOUNDIP, sess,
-						 inet_ntoa (*((struct in_addr *) HostAddr->h_addr)),
-						 NULL, NULL, NULL, 0);
-	}
+
+	signal_emit("server found ip", 2, sess, ip);
 }
 
 void
@@ -1398,12 +1405,6 @@ inbound_user_info (session *sess, char *chan, char *user, char *host,
 					return 0;
 			}
 		}
-	} else
-	{
-		if (!serv->doing_dns)
-			return 0;
-		if (nick && host)
-			do_dns (sess, nick, host);
 	}
 	return 1;
 }
@@ -1432,7 +1433,7 @@ nowindow:
 		if (is_exemption)
 			return FALSE;
 
-		EMIT_SIGNAL (XP_TE_BANLIST, sess, chan, mask, banner, time_str, 0);
+		signal_emit("channel bans", 5, sess, chan, mask, banner, time_str);
 		return TRUE;
 	}
 
@@ -1485,18 +1486,15 @@ inbound_login_end (session *sess, char *text)
 			notify_send_watches (serv);
 		else if(serv->supports_monitor)
 			notify_send_monitor (serv);
-			
+
 		serv->end_of_motd = TRUE;
 	}
 	if (prefs.skipmotd && !serv->motd_skipped)
 	{
 		serv->motd_skipped = TRUE;
-		EMIT_SIGNAL (XP_TE_MOTDSKIP, serv->server_session, NULL, NULL,
-						 NULL, NULL, 0);
 		return;
 	}
-	EMIT_SIGNAL (XP_TE_MOTD, serv->server_session, text, NULL,
-					 NULL, NULL, 0);
+        signal_emit("server motd", 2, serv->server_session, text);
 }
 
 void
