@@ -20,23 +20,19 @@
 
 #define _FILE_OFFSET_BITS 64
 #include <stdio.h>
-#include <unistd.h>
+#include "stdinc.h"
 #include <string.h>
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <pwd.h>
-#include <sys/time.h>
-#include <sys/utsname.h>
 #include <fcntl.h>
-#include <dirent.h>
 #include <errno.h>
 #include "xchat.h"
 #include "xchatc.h"
 #include <ctype.h>
 #include "util.h"
-#include "../../config.h"
+#include "conspire-config.h"
 
 #define WANTSOCKET
 #include "inet.h"
@@ -271,7 +267,7 @@ nocasestrstr (const char *s, const char *wanted)
 
 	if (len == 0)
 		return (char *)s;
-	while (rfc_tolower(*s) != rfc_tolower(*wanted) || strncasecmp (s, wanted, len))
+	while (rfc_tolower(*s) != rfc_tolower(*wanted) || g_ascii_strncasecmp (s, wanted, len))
 		if (*s++ == '\0')
 			return (char *)NULL;
 	return (char *)s;
@@ -320,34 +316,9 @@ waitline (int sok, char *buf, int bufsize, int use_recv)
 char *
 expand_homedir (char *file)
 {
-	char *ret, *user;
-	struct passwd *pw;
-
 	if (*file == '~')
-	{
-		if (file[1] != '\0' && file[1] != '/')
-		{
-			user = strdup(file);
-			if (strchr(user,'/') != NULL)
-				*(strchr(user,'/')) = '\0';
-			if ((pw = getpwnam(user + 1)) == NULL)
-			{
-				free(user);
-				return strdup(file);
-			}
-			free(user);
-			user = strchr(file, '/') != NULL ? strchr(file,'/') : file;
-			ret = malloc(strlen(user) + strlen(pw->pw_dir) + 1);
-			strcpy(ret, pw->pw_dir);
-			strcat(ret, user);
-		}
-		else
-		{
-			ret = malloc (strlen (file) + strlen (g_get_home_dir ()) + 1);
-			sprintf (ret, "%s%s", g_get_home_dir (), file + 1);
-		}
-		return ret;
-	}
+		return g_strdup_printf("%s%s", g_get_home_dir (), file + 1);
+
 	return strdup (file);
 }
 
@@ -526,6 +497,8 @@ get_cpu_info (double *mhz, int *cpus)
 }
 #endif
 
+#ifndef _WIN32
+
 char *
 get_cpu_str (void)
 {
@@ -562,6 +535,52 @@ get_cpu_str (void)
 	return buf;
 }
 
+#else
+
+static int
+get_mhz(void)
+{
+	HKEY hKey;
+	int result, data, dataSize;
+
+	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, "Hardware\\Description\\System\\CentralProcessor\\0", 0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
+	{
+		dataSize = sizeof(data);
+		result = RegQueryValueEx(hKey, "~Mhz", 0, 0, (LPBYTE)&data, &dataSize);
+		RegCloseKey(hKey);
+		if (result == ERROR_SUCCESS)
+			return data;
+	}
+
+	return 0;
+}
+
+char *
+get_cpu_str(void)
+{
+	OSVERSIONINFO osvi;
+	SYSTEM_INFO si;
+	int mhz;
+
+	osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+	GetVersionEx(&osvi);
+	GetSystemInfo(&si);
+
+	mhz = get_mhz();
+	if (mhz)
+	{
+		return g_strdup_printf("Windows %ld.%ld build %ld [i%d86/%.2fMHz]",
+			osvi.dwMajorVersion, osvi.dwMinorVersion, osvi.dwBuildNumber,
+			si.wProcessorLevel, mhz);
+	}
+
+	return g_strdup_printf("Windows %ld.%ld build %ld [i%d86]",
+		osvi.dwMajorVersion, osvi.dwMinorVersion, osvi.dwBuildNumber,
+		si.wProcessorLevel);
+}
+
+#endif
+
 int
 buf_get_line (char *ibuf, char **buf, int *position, int len)
 {
@@ -669,353 +688,26 @@ break_while:
 void
 for_files (char *dirname, char *mask, void callback (char *file))
 {
-	DIR *dir;
-	struct dirent *ent;
+	GDir *dir;
+	const gchar *ent;
 	char *buf;
 
-	dir = opendir (dirname);
+	dir = g_dir_open(dirname, 0, NULL);
 	if (dir)
 	{
-		while ((ent = readdir (dir)))
+		while ((ent = g_dir_read_name(dir)))
 		{
-			if (strcmp (ent->d_name, ".") && strcmp (ent->d_name, ".."))
+			if (match (mask, ent))
 			{
-				if (match (mask, ent->d_name))
-				{
-					buf = malloc (strlen (dirname) + strlen (ent->d_name) + 2);
-					sprintf (buf, "%s/%s", dirname, ent->d_name);
-					callback (buf);
-					free (buf);
-				}
+				buf = g_build_filename(dirname, ent, NULL);				
+				callback(buf);
+				g_free(buf);
 			}
 		}
-		closedir (dir);
+		g_dir_close(dir);
 	}
 }
 
-/*void
-tolowerStr (char *str)
-{
-	while (*str)
-	{
-		*str = rfc_tolower (*str);
-		str++;
-	}
-}*/
-
-typedef struct
-{
-	char *code, *country;
-} domain_t;
-
-static int
-country_compare (const void *a, const void *b)
-{
-	return strcasecmp (a, ((domain_t *)b)->code);
-}
-
-static const domain_t domain[] =
-{
-		{"AC", N_("Ascension Island") },
-		{"AD", N_("Andorra") },
-		{"AE", N_("United Arab Emirates") },
-		{"AF", N_("Afghanistan") },
-		{"AG", N_("Antigua and Barbuda") },
-		{"AI", N_("Anguilla") },
-		{"AL", N_("Albania") },
-		{"AM", N_("Armenia") },
-		{"AN", N_("Netherlands Antilles") },
-		{"AO", N_("Angola") },
-		{"AQ", N_("Antarctica") },
-		{"AR", N_("Argentina") },
-		{"ARPA", N_("Reverse DNS") },
-		{"AS", N_("American Samoa") },
-		{"AT", N_("Austria") },
-		{"ATO", N_("Nato Fiel") },
-		{"AU", N_("Australia") },
-		{"AW", N_("Aruba") },
-		{"AX", N_("Aland Islands") },
-		{"AZ", N_("Azerbaijan") },
-		{"BA", N_("Bosnia and Herzegovina") },
-		{"BB", N_("Barbados") },
-		{"BD", N_("Bangladesh") },
-		{"BE", N_("Belgium") },
-		{"BF", N_("Burkina Faso") },
-		{"BG", N_("Bulgaria") },
-		{"BH", N_("Bahrain") },
-		{"BI", N_("Burundi") },
-		{"BIZ", N_("Businesses"), },
-		{"BJ", N_("Benin") },
-		{"BM", N_("Bermuda") },
-		{"BN", N_("Brunei Darussalam") },
-		{"BO", N_("Bolivia") },
-		{"BR", N_("Brazil") },
-		{"BS", N_("Bahamas") },
-		{"BT", N_("Bhutan") },
-		{"BV", N_("Bouvet Island") },
-		{"BW", N_("Botswana") },
-		{"BY", N_("Belarus") },
-		{"BZ", N_("Belize") },
-		{"CA", N_("Canada") },
-		{"CC", N_("Cocos Islands") },
-		{"CD", N_("Democratic Republic of Congo") },
-		{"CF", N_("Central African Republic") },
-		{"CG", N_("Congo") },
-		{"CH", N_("Switzerland") },
-		{"CI", N_("Cote d'Ivoire") },
-		{"CK", N_("Cook Islands") },
-		{"CL", N_("Chile") },
-		{"CM", N_("Cameroon") },
-		{"CN", N_("China") },
-		{"CO", N_("Colombia") },
-		{"COM", N_("Internic Commercial") },
-		{"CR", N_("Costa Rica") },
-		{"CS", N_("Serbia and Montenegro") },
-		{"CU", N_("Cuba") },
-		{"CV", N_("Cape Verde") },
-		{"CX", N_("Christmas Island") },
-		{"CY", N_("Cyprus") },
-		{"CZ", N_("Czech Republic") },
-		{"DE", N_("Germany") },
-		{"DJ", N_("Djibouti") },
-		{"DK", N_("Denmark") },
-		{"DM", N_("Dominica") },
-		{"DO", N_("Dominican Republic") },
-		{"DZ", N_("Algeria") },
-		{"EC", N_("Ecuador") },
-		{"EDU", N_("Educational Institution") },
-		{"EE", N_("Estonia") },
-		{"EG", N_("Egypt") },
-		{"EH", N_("Western Sahara") },
-		{"ER", N_("Eritrea") },
-		{"ES", N_("Spain") },
-		{"ET", N_("Ethiopia") },
-		{"EU", N_("European Union") },
-		{"FI", N_("Finland") },
-		{"FJ", N_("Fiji") },
-		{"FK", N_("Falkland Islands") },
-		{"FM", N_("Micronesia") },
-		{"FO", N_("Faroe Islands") },
-		{"FR", N_("France") },
-		{"GA", N_("Gabon") },
-		{"GB", N_("Great Britain") },
-		{"GD", N_("Grenada") },
-		{"GE", N_("Georgia") },
-		{"GF", N_("French Guiana") },
-		{"GG", N_("British Channel Isles") },
-		{"GH", N_("Ghana") },
-		{"GI", N_("Gibraltar") },
-		{"GL", N_("Greenland") },
-		{"GM", N_("Gambia") },
-		{"GN", N_("Guinea") },
-		{"GOV", N_("Government") },
-		{"GP", N_("Guadeloupe") },
-		{"GQ", N_("Equatorial Guinea") },
-		{"GR", N_("Greece") },
-		{"GS", N_("S. Georgia and S. Sandwich Isles") },
-		{"GT", N_("Guatemala") },
-		{"GU", N_("Guam") },
-		{"GW", N_("Guinea-Bissau") },
-		{"GY", N_("Guyana") },
-		{"HK", N_("Hong Kong") },
-		{"HM", N_("Heard and McDonald Islands") },
-		{"HN", N_("Honduras") },
-		{"HR", N_("Croatia") },
-		{"HT", N_("Haiti") },
-		{"HU", N_("Hungary") },
-		{"ID", N_("Indonesia") },
-		{"IE", N_("Ireland") },
-		{"IL", N_("Israel") },
-		{"IM", N_("Isle of Man") },
-		{"IN", N_("India") },
-		{"INFO", N_("Informational") },
-		{"INT", N_("International") },
-		{"IO", N_("British Indian Ocean Territory") },
-		{"IQ", N_("Iraq") },
-		{"IR", N_("Iran") },
-		{"IS", N_("Iceland") },
-		{"IT", N_("Italy") },
-		{"JE", N_("Jersey") },
-		{"JM", N_("Jamaica") },
-		{"JO", N_("Jordan") },
-		{"JP", N_("Japan") },
-		{"KE", N_("Kenya") },
-		{"KG", N_("Kyrgyzstan") },
-		{"KH", N_("Cambodia") },
-		{"KI", N_("Kiribati") },
-		{"KM", N_("Comoros") },
-		{"KN", N_("St. Kitts and Nevis") },
-		{"KP", N_("North Korea") },
-		{"KR", N_("South Korea") },
-		{"KW", N_("Kuwait") },
-		{"KY", N_("Cayman Islands") },
-		{"KZ", N_("Kazakhstan") },
-		{"LA", N_("Laos") },
-		{"LB", N_("Lebanon") },
-		{"LC", N_("Saint Lucia") },
-		{"LI", N_("Liechtenstein") },
-		{"LK", N_("Sri Lanka") },
-		{"LR", N_("Liberia") },
-		{"LS", N_("Lesotho") },
-		{"LT", N_("Lithuania") },
-		{"LU", N_("Luxembourg") },
-		{"LV", N_("Latvia") },
-		{"LY", N_("Libya") },
-		{"MA", N_("Morocco") },
-		{"MC", N_("Monaco") },
-		{"MD", N_("Moldova") },
-		{"MED", N_("United States Medical") },
-		{"MG", N_("Madagascar") },
-		{"MH", N_("Marshall Islands") },
-		{"MIL", N_("Military") },
-		{"MK", N_("Macedonia") },
-		{"ML", N_("Mali") },
-		{"MM", N_("Myanmar") },
-		{"MN", N_("Mongolia") },
-		{"MO", N_("Macau") },
-		{"MP", N_("Northern Mariana Islands") },
-		{"MQ", N_("Martinique") },
-		{"MR", N_("Mauritania") },
-		{"MS", N_("Montserrat") },
-		{"MT", N_("Malta") },
-		{"MU", N_("Mauritius") },
-		{"MV", N_("Maldives") },
-		{"MW", N_("Malawi") },
-		{"MX", N_("Mexico") },
-		{"MY", N_("Malaysia") },
-		{"MZ", N_("Mozambique") },
-		{"NA", N_("Namibia") },
-		{"NC", N_("New Caledonia") },
-		{"NE", N_("Niger") },
-		{"NET", N_("Internic Network") },
-		{"NF", N_("Norfolk Island") },
-		{"NG", N_("Nigeria") },
-		{"NI", N_("Nicaragua") },
-		{"NL", N_("Netherlands") },
-		{"NO", N_("Norway") },
-		{"NP", N_("Nepal") },
-		{"NR", N_("Nauru") },
-		{"NU", N_("Niue") },
-		{"NZ", N_("New Zealand") },
-		{"OM", N_("Oman") },
-		{"ORG", N_("Internic Non-Profit Organization") },
-		{"PA", N_("Panama") },
-		{"PE", N_("Peru") },
-		{"PF", N_("French Polynesia") },
-		{"PG", N_("Papua New Guinea") },
-		{"PH", N_("Philippines") },
-		{"PK", N_("Pakistan") },
-		{"PL", N_("Poland") },
-		{"PM", N_("St. Pierre and Miquelon") },
-		{"PN", N_("Pitcairn") },
-		{"PR", N_("Puerto Rico") },
-		{"PS", N_("Palestinian Territory") },
-		{"PT", N_("Portugal") },
-		{"PW", N_("Palau") },
-		{"PY", N_("Paraguay") },
-		{"QA", N_("Qatar") },
-		{"RE", N_("Reunion") },
-		{"RO", N_("Romania") },
-		{"RPA", N_("Old School ARPAnet") },
-		{"RU", N_("Russian Federation") },
-		{"RW", N_("Rwanda") },
-		{"SA", N_("Saudi Arabia") },
-		{"SB", N_("Solomon Islands") },
-		{"SC", N_("Seychelles") },
-		{"SD", N_("Sudan") },
-		{"SE", N_("Sweden") },
-		{"SG", N_("Singapore") },
-		{"SH", N_("St. Helena") },
-		{"SI", N_("Slovenia") },
-		{"SJ", N_("Svalbard and Jan Mayen Islands") },
-		{"SK", N_("Slovak Republic") },
-		{"SL", N_("Sierra Leone") },
-		{"SM", N_("San Marino") },
-		{"SN", N_("Senegal") },
-		{"SO", N_("Somalia") },
-		{"SR", N_("Suriname") },
-		{"ST", N_("Sao Tome and Principe") },
-		{"SU", N_("Former USSR") },
-		{"SV", N_("El Salvador") },
-		{"SY", N_("Syria") },
-		{"SZ", N_("Swaziland") },
-		{"TC", N_("Turks and Caicos Islands") },
-		{"TD", N_("Chad") },
-		{"TF", N_("French Southern Territories") },
-		{"TG", N_("Togo") },
-		{"TH", N_("Thailand") },
-		{"TJ", N_("Tajikistan") },
-		{"TK", N_("Tokelau") },
-		{"TL", N_("East Timor") },
-		{"TM", N_("Turkmenistan") },
-		{"TN", N_("Tunisia") },
-		{"TO", N_("Tonga") },
-		{"TP", N_("East Timor") },
-		{"TR", N_("Turkey") },
-		{"TT", N_("Trinidad and Tobago") },
-		{"TV", N_("Tuvalu") },
-		{"TW", N_("Taiwan") },
-		{"TZ", N_("Tanzania") },
-		{"UA", N_("Ukraine") },
-		{"UG", N_("Uganda") },
-		{"UK", N_("United Kingdom") },
-		{"US", N_("United States of America") },
-		{"UY", N_("Uruguay") },
-		{"UZ", N_("Uzbekistan") },
-		{"VA", N_("Vatican City State") },
-		{"VC", N_("St. Vincent and the Grenadines") },
-		{"VE", N_("Venezuela") },
-		{"VG", N_("British Virgin Islands") },
-		{"VI", N_("US Virgin Islands") },
-		{"VN", N_("Vietnam") },
-		{"VU", N_("Vanuatu") },
-		{"WF", N_("Wallis and Futuna Islands") },
-		{"WS", N_("Samoa") },
-		{"YE", N_("Yemen") },
-		{"YT", N_("Mayotte") },
-		{"YU", N_("Yugoslavia") },
-		{"ZA", N_("South Africa") },
-		{"ZM", N_("Zambia") },
-		{"ZW", N_("Zimbabwe") },
-};
-
-char *
-country (char *hostname)
-{
-	char *p;
-	domain_t *dom;
-
-	if (!hostname || !*hostname || isdigit ((unsigned char) hostname[strlen (hostname) - 1]))
-		return _("Unknown");
-	if ((p = strrchr (hostname, '.')))
-		p++;
-	else
-		p = hostname;
-
-	dom = bsearch (p, domain, sizeof (domain) / sizeof (domain_t),
-						sizeof (domain_t), country_compare);
-
-	if (!dom)
-		return _("Unknown");
-
-	return _(dom->country);
-}
-
-void
-country_search (char *pattern, void *ud, void (*print)(void *, char *, ...))
-{
-	const domain_t *dom;
-	int i;
-
-	for (i = 0; i < sizeof (domain) / sizeof (domain_t); i++)
-	{
-		dom = &domain[i];
-		if (match (pattern, dom->country) || match (pattern, _(dom->country)))
-		{
-			print (ud, "%s = %s\n", dom->code, _(dom->country));
-		}
-	}
-}
 
 /* I think gnome1.0.x isn't necessarily linked against popt, ah well! */
 /* !!! For now use this inlined function, or it would break fe-text building */
@@ -1115,14 +807,16 @@ int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
 int
 util_exec (const char *cmd)
 {
-	int pid;
+#ifndef _WIN32
+	int pid, fd;
+#endif
 	char **argv;
 	int argc;
-	int fd;
 
 	if (my_poptParseArgvString (cmd, &argc, &argv) != 0)
 		return -1;
 
+#ifndef _WIN32
 	pid = fork ();
 	if (pid == -1)
 		return -1;
@@ -1132,16 +826,23 @@ util_exec (const char *cmd)
 		for (fd = 3; fd < 1024; fd++) close(fd);
 		execvp (argv[0], argv);
 		_exit (0);
-	} else
+	}
+	else
 	{
 		free (argv);
 		return pid;
 	}
+#else
+	_spawnvp(_P_DETACH, argv[0], argv);
+	free(argv);
+	return 0;
+#endif
 }
 
 int
 util_execv (const char * const argv[])
 {
+#ifndef _WIN32
 	int pid, fd;
 
 	pid = fork ();
@@ -1157,6 +858,10 @@ util_execv (const char * const argv[])
 	{
 		return pid;
 	}
+#else
+	_spawnv(_P_DETACH, argv[0], argv);
+	return 0;
+#endif
 }
 
 unsigned long
@@ -1416,6 +1121,10 @@ move_file_utf8 (char *src_dir, char *dst_dir, char *fname, int dccpermissions)
 	char *src_fs;	/* FileSystem encoding */
 	char *dst_fs;
 
+	g_return_if_fail(src_dir != NULL);
+	g_return_if_fail(dst_dir != NULL);
+	g_return_if_fail(fname != NULL);
+
 	/* if dcc_dir and dcc_completed_dir are the same then we are done */
 	if (0 == strcmp (src_dir, dst_dir) ||
 		 0 == dst_dir[0])
