@@ -22,22 +22,16 @@
 #include <time.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#include "stdinc.h"
 
 #define WANTSOCKET
 #include "inet.h"
 
-#include <sys/wait.h>
-#include <signal.h>
-
 #include "xchat.h"
 #include "fe.h"
 #include "util.h"
 #include "cfgfiles.h"
-#include "ignore.h"
-#include "xchat-plugin.h"
-#include "plugin.h"
-#include "plugin-timer.h"
+#include "ignore-ng.h"
 #include "notify.h"
 #include "server.h"
 #include "servlist.h"
@@ -45,6 +39,10 @@
 #include "text.h"
 #include "url.h"
 #include "xchatc.h"
+#include "proto-irc.h"
+#include "upnp.h"
+#include "sasl.h"
+#include "plugin.h"
 
 GSList *popup_list = 0;
 GSList *button_list = 0;
@@ -62,6 +60,23 @@ GSList *tabmenu_list = 0;
 
 //ConfigDb *config;
 
+/*
+ * This array contains 5 double linked lists, one for each priority in the
+ * "interesting session" queue ("channel" stands for everything but
+ * SESS_DIALOG):
+ *
+ * [0] queries with hilight
+ * [1] queries
+ * [2] channels with hilight
+ * [3] channels with dialogue
+ * [4] channels with other data
+ *
+ * Each time activity happens the corresponding session is put at the
+ * beginning of one of the lists.  The aim is to be able to switch to the
+ * session with the most important/recent activity.
+ */
+GList *sess_list_by_lastact[5] = {NULL, NULL, NULL, NULL, NULL};
+
 static int in_xchat_exit = FALSE;
 int xchat_is_quitting = FALSE;
 /* command-line args */
@@ -70,11 +85,6 @@ int arg_skip_plugins = FALSE;
 char *arg_url = NULL;
 gint arg_existing = FALSE;
 
-#ifdef USE_DBUS
-#include "dbus/dbus-client.h"
-#include "dbus/dbus-plugin.h"
-#endif /* USE_DBUS */
-
 struct session *current_tab;
 struct session *current_sess = 0;
 struct xchatprefs prefs;
@@ -127,7 +137,7 @@ lagcheck_update (void)
 {
 	server *serv;
 	GSList *list = serv_list;
-	
+
 	if (!prefs.lagometer)
 		return;
 
@@ -161,10 +171,7 @@ lag_check (void)
 			lag = now - serv->ping_recv;
 			if (prefs.pingtimeout && lag > prefs.pingtimeout && lag > 0)
 			{
-				sprintf (tbuf, "%d", lag);
-				EMIT_SIGNAL (XP_TE_PINGTIMEOUT, serv->server_session, tbuf, NULL,
-								 NULL, NULL, 0);
-				serv->auto_reconnect (serv, FALSE, -1);
+				signal_emit("server stoned", 2, serv, lag);
 			} else
 			{
 				snprintf (tbuf, sizeof (tbuf), "LAG%lu", tim);
@@ -272,16 +279,8 @@ irc_init (session *sess)
 
 	done_init = TRUE;
 
-#ifdef USE_PLUGIN
 	if (!arg_skip_plugins)
-		plugin_auto_load (sess);	/* autoload ~/.xchat *.so */
-#endif
-
-#ifdef USE_DBUS
-	plugin_add (sess, NULL, NULL, dbus_plugin_init, NULL, NULL, FALSE);
-#endif
-
-	plugin_add (sess, NULL, NULL, timer_plugin_init, NULL, NULL, FALSE);
+		plugin_autoload();	/* autoload ~/.xchat *.so */
 
 	if (prefs.notify_timeout)
 		notify_tag = g_timeout_add (prefs.notify_timeout * 1000, (GSourceFunc) notify_checklist, 0);
@@ -317,6 +316,11 @@ session_new (server *serv, char *from, int type, int focus)
 
 	sess_list = g_slist_prepend (sess_list, sess);
 
+	sess->lastact_elem = NULL;
+	sess->lastact_idx = LACT_NONE;
+
+	signal_emit("session create", 1, sess);
+
 	fe_new_window (sess, focus);
 
 	return sess;
@@ -354,29 +358,10 @@ new_ircwindow (server *serv, char *name, int type, int focus)
 	irc_init (sess);
 	if (prefs.text_replay)
 		scrollback_load (sess);
-	plugin_emit_dummy_print (sess, "Open Context");
 
 	return sess;
 }
 
-static void
-exec_notify_kill (session * sess)
-{
-	struct nbexec *re;
-	if (sess->running_exec != NULL)
-	{
-		re = sess->running_exec;
-		sess->running_exec = NULL;
-		kill (re->childpid, SIGKILL);
-		waitpid (re->childpid, NULL, WNOHANG);
-		g_source_remove (re->iotag);
-		close (re->myfd);
-		if (re->linebuf)
-			free(re->linebuf);
-		free (re);
-	}
-}
-
 static void
 send_quit_or_part (session * killsess)
 {
@@ -407,7 +392,7 @@ send_quit_or_part (session * killsess)
 		{
 			if (!killserv->sent_quit)
 			{
-				killserv->flush_queue (killserv);
+				linequeue_erase(killserv->lq);
 				server_sendquit (killsess);
 				killserv->sent_quit = TRUE;
 			}
@@ -429,8 +414,6 @@ session_free (session *killsess)
 	session *sess;
 	GSList *list;
 
-	plugin_emit_dummy_print (killsess, "Close Context");
-
 	if (current_tab == killsess)
 		current_tab = NULL;
 
@@ -461,11 +444,11 @@ session_free (session *killsess)
 
 	sess_list = g_slist_remove (sess_list, killsess);
 
+	signal_emit("session destroy", 1, killsess);
+
 	if (killsess->type == SESS_CHANNEL)
 		userlist_free (killsess);
 
-	exec_notify_kill (killsess);
-
 	log_close (killsess);
 	scrollback_close (killsess);
 
@@ -486,6 +469,14 @@ session_free (session *killsess)
 			current_sess = sess_list->data;
 	}
 
+	if (killsess->lastact_elem)
+	{
+		if (killsess->lastact_idx != LACT_NONE)
+			sess_list_by_lastact[killsess->lastact_idx] = g_list_delete_link(sess_list_by_lastact[killsess->lastact_idx], killsess->lastact_elem);
+		else
+			g_list_free_1(killsess->lastact_elem);
+	}
+
 	free (killsess);
 
 	if (!sess_list && !in_xchat_exit)
@@ -515,6 +506,113 @@ free_sessions (void)
 	}
 }
 
+/*
+ * Update the priority queue of the "interesting sessions"
+ * (sess_list_by_lastact).
+ */
+void
+lastact_update(session *sess)
+{
+	int newidx;
+
+	/*
+	 * Find the priority (for the order see before).
+	 *  NOTE: we ignore new_data, because (especially with a lot of windows),
+	 *  that makes lastact essentially useless and annoying.
+	 */
+	if (sess->type == SESS_DIALOG)
+	{
+		if (sess->nick_said)
+			newidx = LACT_QUERY_HI;
+		else if (sess->msg_said)
+			newidx = LACT_QUERY;
+/*
+		else if (sess->new_data)
+			newidx = LACT_QUERY;
+*/
+		else
+			newidx = LACT_NONE;
+	}
+	else
+	{
+		if (sess->nick_said)
+			newidx = LACT_CHAN_HI;
+		else if (sess->msg_said)
+			newidx = LACT_CHAN;
+/*
+		else if (sess->new_data)
+			newidx = LACT_CHAN_DATA;
+*/
+		else
+			newidx = LACT_NONE;
+	}
+
+	/* Check if this update is a no-op */
+	if (sess->lastact_idx == newidx &&
+			((newidx != LACT_NONE && sess->lastact_elem == sess_list_by_lastact[newidx]) ||
+			 (newidx == LACT_NONE)))
+		return;
+
+	/* Remove from the old position (and, if no new position, return */
+	else if (sess->lastact_idx != LACT_NONE && sess->lastact_elem)
+	{
+		sess_list_by_lastact[sess->lastact_idx] = g_list_remove_link(
+				sess_list_by_lastact[sess->lastact_idx],
+				sess->lastact_elem);
+		if (newidx == LACT_NONE)
+		{
+			sess->lastact_idx = newidx;
+			return;
+		}
+	}
+
+	/* No previous position, allocate new */
+	else if (!sess->lastact_elem)
+		sess->lastact_elem = g_list_prepend(sess->lastact_elem, sess);
+
+	sess->lastact_idx = newidx;
+	sess_list_by_lastact[newidx] = g_list_concat(
+			sess->lastact_elem, sess_list_by_lastact[newidx]);
+}
+
+/*
+ * Extract the first session from the priority queue of sessions with recent
+ * activity. Return NULL if no such session can be found.
+ *
+ * If filter is specified, skip a session if filter(session) returns 0. This
+ * can be used for UI-specific needs, e.g. in fe-gtk we want to filter out
+ * detached sessions.
+ */
+session *
+lastact_getfirst(int (*filter) (session *sess))
+{
+	int i;
+	session *sess = NULL;
+	GList *curitem;
+
+	/* 5 is the number of priority classes LACT_ */
+	for (i = 0; i < 5 && !sess; i++)
+	{
+		curitem = sess_list_by_lastact[i];
+		while (curitem && !sess)
+		{
+			sess = g_list_nth_data(curitem, 0);
+			if (!sess || (filter && !filter(sess)))
+			{
+				sess = NULL;
+				curitem = g_list_next(curitem);
+			}
+		}
+
+		if (sess)
+		{
+			sess_list_by_lastact[i] = g_list_remove_link(sess_list_by_lastact[i], curitem);
+			sess->lastact_idx = LACT_NONE;
+		}
+	}
+
+	return sess;
+}
 
 
 #define XTERM "gnome-terminal -x "
@@ -532,32 +630,29 @@ static char defaultconf_regex_replace[] =
 #endif
 
 static char defaultconf_commands[] =
-	"NAME ACTION\n"         "CMD me &2\n\n"\
-	"NAME AME\n"            "CMD allchan me &2\n\n"\
-	"NAME ANICK\n"          "CMD allserv nick &2\n\n"\
-	"NAME AMSG\n"           "CMD allchan say &2\n\n"\
+	"NAME ACTION\n"         "CMD describe $2 $3-\n\n"\
 	"NAME BANLIST\n"        "CMD quote MODE %c +b\n\n"\
-	"NAME CHAT\n"           "CMD dcc chat %2\n\n"\
-	"NAME DIALOG\n"         "CMD query %2\n\n"\
-	"NAME DMSG\n"           "CMD msg =%2 &3\n\n"\
-	"NAME GREP\n"           "CMD lastlog -r &2\n\n"\
-	"NAME J\n"              "CMD join &2\n\n"\
-	"NAME KILL\n"           "CMD quote KILL %2 :&3\n\n"\
-	"NAME LEAVE\n"          "CMD part &2\n\n"\
-	"NAME M\n"              "CMD msg &2\n\n"\
-	"NAME ONOTICE\n"        "CMD notice @%c &2\n\n"\
-	"NAME RAW\n"            "CMD quote &2\n\n"\
+	"NAME CHAT\n"           "CMD dcc chat $2\n\n"\
+	"NAME DIALOG\n"         "CMD query $2\n\n"\
+	"NAME DMSG\n"           "CMD msg =$2 $3-\n\n"\
+	"NAME GREP\n"           "CMD lastlog -r $2-\n\n"\
+	"NAME J\n"              "CMD join $2-\n\n"\
+	"NAME KILL\n"           "CMD quote KILL $2 :$3-\n\n"\
+	"NAME LEAVE\n"          "CMD part $2-\n\n"\
+	"NAME M\n"              "CMD msg $2-\n\n"\
+	"NAME ONOTICE\n"        "CMD notice @%c $2-\n\n"\
+	"NAME RAW\n"            "CMD quote $2-\n\n"\
 	"NAME SERVHELP\n"       "CMD quote HELP\n\n"\
 	"NAME SPING\n"          "CMD ping\n\n"\
-	"NAME SQUERY\n"         "CMD quote SQUERY %2 :&3\n\n"\
-	"NAME SSLSERVER\n"      "CMD server -ssl &2\n\n"\
+	"NAME SQUERY\n"         "CMD quote SQUERY $2 :$3-\n\n"\
+	"NAME SSLSERVER\n"      "CMD server -ssl $2-\n\n"\
 	"NAME SV\n"             "CMD echo xchat %v %m\n\n"\
-	"NAME UMODE\n"          "CMD mode %n &2\n\n"\
+	"NAME UMODE\n"          "CMD mode %n $2-\n\n"\
 	"NAME UPTIME\n"         "CMD quote STATS u\n\n"\
-	"NAME VER\n"            "CMD ctcp %2 VERSION\n\n"\
-	"NAME VERSION\n"        "CMD ctcp %2 VERSION\n\n"\
-	"NAME WALLOPS\n"        "CMD quote WALLOPS :&2\n\n"\
-	"NAME WII\n"            "CMD quote WHOIS %2 %2\n\n";
+	"NAME VER\n"            "CMD ctcp $2 VERSION\n\n"\
+	"NAME VERSION\n"        "CMD ctcp $2 VERSION\n\n"\
+	"NAME WALLOPS\n"        "CMD quote WALLOPS :$2-\n\n"\
+	"NAME WII\n"            "CMD quote WHOIS $2 $2\n\n";
 
 static char defaultconf_urlhandlers[] =
 	"NAME SUB\n"                            "CMD Epiphany...\n\n"\
@@ -605,12 +700,12 @@ static char defaultconf_urlhandlers[] =
 		"NAME Telnet\n"                     "CMD !"XTERM"telnet %s\n\n"\
 		"NAME Ping\n"                       "CMD !"XTERM"ping -c 4 %s\n\n"\
 	"NAME ENDSUB\n"                         "CMD \n\n"\
-	"NAME Connect as IRC server\n"          "CMD url %s\n\n";
+	"NAME Connect to IRC server\n"          "CMD newserver %s\n\n";
 
 #ifdef USE_SIGACTION
 /* Close and open log files on SIGUSR1. Usefull for log rotating */
 
-static void 
+static void
 sigusr1_handler (int signal, siginfo_t *si, void *un)
 {
 	GSList *list = sess_list;
@@ -673,9 +768,14 @@ xchat_init (void)
 	if (g_get_charset (&cs))
 		prefs.utf8_locale = TRUE;
 
+	command_init ();
 	load_text_events ();
 	notify_load ();
 	ignore_load ();
+	signal_printer_init();
+	proto_irc_init();
+	upnp_init();
+	sasl_init();
 
 	snprintf (buf, sizeof (buf),
 		"NAME %s\n"                         "CMD query %%s\n\n"\
@@ -801,7 +901,9 @@ xchat_exit (void)
 {
 	xchat_is_quitting = TRUE;
 	in_xchat_exit = TRUE;
-	plugin_kill_all ();
+#if 0
+	plugin_shutdown();
+#endif
 	fe_cleanup ();
 	if (prefs.autosave)
 	{
@@ -820,33 +922,57 @@ xchat_exit (void)
 static int
 child_handler (gpointer userdata)
 {
+#ifndef _WIN32
 	int pid = GPOINTER_TO_INT (userdata);
 
 	if (waitpid (pid, 0, WNOHANG) == pid)
 		return 0;					  /* remove timeout handler */
-	return 1;						  /* keep the timeout handler */
+	return 1; /* keep the timeout handler */
+#endif
 }
 
 void
 xchat_exec (const char *cmd)
 {
+#ifndef _WIN32
 	int pid = util_exec (cmd);
 	if (pid != -1)
 	/* zombie avoiding system. Don't ask! it has to be like this to work
       with zvt (which overrides the default handler) */
 		g_timeout_add (1000, (GSourceFunc) child_handler, GINT_TO_POINTER (pid));
+#endif
 }
 
 void
 xchat_execv (const char * const argv[])
 {
+#ifndef _WIN32
 	int pid = util_execv (argv);
 	if (pid != -1)
 	/* zombie avoiding system. Don't ask! it has to be like this to work
       with zvt (which overrides the default handler) */
 		g_timeout_add (1000, (GSourceFunc) child_handler, GINT_TO_POINTER (pid));
+#endif
 }
 
+#ifdef _WIN32
+void
+conspire_init_winsock(void)
+{
+	WORD wVersionRequested;
+	WSADATA wsaData;
+	int err;
+
+	wVersionRequested = MAKEWORD(2, 2);
+
+	err = WSAStartup(wVersionRequested, &wsaData);
+	if (err) {
+		g_print("Winsock failed to initialize, aborting: %d\n", err);
+		exit(EXIT_FAILURE);
+	}
+}
+#endif
+
 int
 main (int argc, char *argv[])
 {
@@ -855,10 +981,14 @@ main (int argc, char *argv[])
 	g_thread_init(NULL);
 	mowgli_init();
 
+#ifdef _WIN32
+	conspire_init_winsock();
+#endif
+
 #ifdef GNUTLS
 	gnutls_global_init ();
 #endif
-	
+
 	srand (time (0));	/* CL: do this only once! */
 
 #ifdef SOCKS
@@ -868,10 +998,6 @@ main (int argc, char *argv[])
 	ret = fe_args (argc, argv);
 	if (ret != -1)
 		return ret;
-	
-#ifdef USE_DBUS
-	xchat_remote ();
-#endif
 
 	load_config ();
 
