@@ -1,4 +1,3 @@
-#include "../../config.h"
 
 #include <glib/gslist.h>
 #include <glib/glist.h>
@@ -11,16 +10,32 @@
 #ifndef XCHAT_H
 #define XCHAT_H
 
+#ifdef _MSC_VER
+# pragma warning (disable: 4996)
+# pragma warning (disable: 4005)
+#endif
+
+#include "stdinc.h"
 #include "debug.h"
 #include "history.h"
+#include "signal_factory.h"
+#include "signal_printer.h"
+#include "cap.h"
+#include "linequeue.h"
+
+#ifdef _CONSPIRE_INTERNAL
+#include "conspire-config.h"
 
 #ifndef HAVE_SNPRINTF
+#undef snprintf
 #define snprintf g_snprintf
 #endif
 
 #ifndef HAVE_VSNPRINTF
+#undef vsnprintf
 #define vsnprintf g_vsnprintf
 #endif
+#endif
 
 #ifdef GNUTLS
 #include <gnutls/gnutls.h>
@@ -85,17 +100,6 @@ void *xchat_realloc (char *old, int len, char *file, int line);
 
 #include <mowgli.h>
 
-struct nbexec
-{
-	int myfd;
-	int childpid;
-	int tochannel;					  /* making this int keeps the struct 4-byte aligned */
-	int iotag;
-	char *linebuf;
-	int buffill;
-	struct session *sess;
-};
-
 struct xchatprefs
 {
 	char *nick1;
@@ -127,6 +131,7 @@ struct xchatprefs
 
 	char *text_overflow_start;
 	char *text_overflow_stop;
+	gint *text_overflow_limit;
 
 	char *proxy_host;
 	int proxy_port;
@@ -304,6 +309,7 @@ struct xchatprefs
 
 	gboolean redundant_nickstamps;
 	gboolean strip_quits;
+        gboolean hilight_enable;
 };
 
 /* Session types */
@@ -313,6 +319,15 @@ struct xchatprefs
 #define SESS_NOTICES	4
 #define SESS_SNOTICES	5
 
+/* Priorities in the "interesting sessions" priority queue
+ * (see xchat.c:sess_list_by_lastact) */
+#define LACT_NONE		-1		/* no queues */
+#define LACT_QUERY_HI	0		/* query with hilight */
+#define LACT_QUERY		1		/* query with messages */
+#define LACT_CHAN_HI	2		/* channel with hilight */
+#define LACT_CHAN		3		/* channel with messages */
+#define LACT_CHAN_DATA	4		/* channel with other data */
+
 typedef struct session
 {
 	struct server *server;
@@ -329,6 +344,7 @@ typedef struct session
 	int scrollwritten;					/* number of lines written */
 
 	char lastnick[NICKLEN];			  /* last nick you /msg'ed */
+	GSList *split_list;					/* a list of netsplitted users for this tab */
 
 	struct history history;
 
@@ -344,13 +360,17 @@ typedef struct session
 	int mode_timeout_tag;
 
 	struct session *lastlog_sess;
-	struct nbexec *running_exec;
 
 	struct session_gui *gui;		/* initialized by fe_new_window */
 	struct restore_gui *res;
 
 	int userlisthidden;
-	int type;
+	int type; /* SESS_ */
+
+	GList *lastact_elem;    /* our GList element in sess_list_by_lastact */
+	int lastact_idx;                /* the sess_list_by_lastact[] index of the list we're in.
+					 * For valid values, see defines of LACT_*. */
+
 	int new_data:1;			/* new data avail? (purple tab) */
 	int nick_said:1;		/* your nick mentioned? (blue tab) */
 	int msg_said:1;			/* new msg available? (red tab) */
@@ -367,8 +387,11 @@ typedef struct session
 	int done_away_check:1;	/* done checking for away status changes */
 	unsigned int lastlog_regexp:1;	/* this is a lastlog and using regexp */
 	gboolean immutable;	/* don't close me. */
+	gboolean ul_blocked;	/* don't update the GUI */
 } session;
 
+#include "session.h"
+
 struct msproxy_state_t
 {
 	gint32				clientid;
@@ -463,14 +486,9 @@ typedef struct server
 
 	void *network;						/* points to entry in servlist.c or NULL! */
 
-	GSList *outbound_queue;
-	time_t next_send;						/* cptr->since in ircu */
-	time_t prev_now;					/* previous now-time */
-	int sendq_len;						/* queue size */
 	int lag;								/* milliseconds */
 
 	/* netsplit detection. --nenolod */
-	GSList *split_list;
 	guint split_timer;
 	char *split_reason;
 	char *split_serv1;
@@ -509,7 +527,7 @@ typedef struct server
 	int dont_use_proxy:1;		/* to proxy or not to proxy */
 	int supports_watch:1;		/* supports the WATCH command */
 	int supports_monitor:1;		/* supports the MONITOR command */
-	int inside_monitor:1;	
+	int inside_monitor:1;
 	int bad_prefix:1;				/* gave us a bad PREFIX= 005 number */
 	unsigned int have_namesx:1;	/* 005 tokens NAMESX and UHNAMES */
 	unsigned int have_uhnames:1;
@@ -529,15 +547,19 @@ typedef struct server
 	char *sasl_pass;
 	SaslState sasl_state;
 	int sasl_timeout_tag;
+	CapState *cap;
+	LineQueue *lq;
 } server;
 
 typedef int (*cmd_callback) (struct session * sess, char *tbuf, char *word[],
 									  char *word_eol[]);
 
+#include "command_factory.h"
+
 struct commands
 {
 	char *name;
-	cmd_callback callback;
+	CommandHandler callback;
 	char needserver;
 	char needchannel;
 	gint16 handle_quotes;
