@@ -20,7 +20,7 @@
 #include <stdlib.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#include "../common/stdinc.h"
 #include <string.h>
 #include <fcntl.h>
 #include <ctype.h>
@@ -48,7 +48,6 @@
 #include "../common/outbound.h"
 #include "../common/util.h"
 #include "../common/text.h"
-#include "../common/plugin.h"
 #include <gdk/gdkkeysyms.h>
 #include "gtkutil.h"
 #include "menu.h"
@@ -58,10 +57,6 @@
 #include "textgui.h"
 #include "fkeys.h"
 
-#ifdef USE_GTKSPELL
-#include <gtk/gtktextview.h>
-#endif
-
 static void replace_handle (GtkWidget * wid);
 void key_action_tab_clean (void);
 
@@ -158,7 +153,7 @@ static const struct key_action key_actions[KEY_MAX_ACTIONS + 1] = {
 	{key_action_handle_command, "Run Command",
 	 N_("The \002Run Command\002 action runs the data in Data 1 as if it has been typed into the entry box where you pressed the key sequence. Thus it can contain text (which will be sent to the channel/person), commands or user commands. When run all \002\\n\002 characters in Data 1 are used to deliminate seperate commands so it is possible to run more than one command. If you want a \002\\\002 in the actual text run then enter \002\\\\\002")},
 	{key_action_page_switch, "Change Page",
-	 N_("The \002Change Page\002 command switches between pages in the notebook. Set Data 1 to the page you want to switch to. If Data 2 is set to anything then the switch will be relative to the current position")},
+	 N_("The \002Change Page\002 command switches between pages in the notebook. Set Data 1 to the page you want to switch to. If Data 2 is set to anything then the switch will be relative to the current position. Set Data 1 to auto to switch to the page with the most recent and important activity (queries first, then channels with hilight, channels with dialogue, channels with other data)")},
 	{key_action_insert, "Insert in Buffer",
 	 N_("The \002Insert in Buffer\002 command will insert the contents of Data 1 into the entry where the key sequence was pressed at the current cursor position")},
 	{key_action_scroll_page, "Scroll Page",
@@ -231,10 +226,10 @@ key_get_key_name (int keyval)
    * key bindings are stored in a linked list of key_binding structs
    * which looks like {
    int keyval;  GDK keynumber
-   char *keyname;  String with the name of the function 
-   int action;  Index into key_actions 
-   int mod; Flags of STATE_* above 
-   char *data1, *data2;  Pointers to strings, these must be freed 
+   char *keyname;  String with the name of the function
+   int action;  Index into key_actions
+   int mod; Flags of STATE_* above
+   char *data1, *data2;  Pointers to strings, these must be freed
    struct key_binding *next;
    }
    * remember that is (data1 || data2) != NULL then they need to be free()'ed
@@ -268,8 +263,10 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 		return FALSE;
 	current_sess = sess;
 
+#if 0
 	if (plugin_emit_keypress (sess, evt->state, evt->keyval, evt->length, evt->string))
 		return 1;
+#endif
 
 	/* maybe the plugin closed this tab? */
 	if (!is_session (sess))
@@ -314,17 +311,6 @@ key_handle_key_press (GtkWidget *wid, GdkEventKey *evt, session *sess)
 	case GDK_space:
 		key_action_tab_clean ();
 		break;
-
-#ifdef USE_GTKSPELL
-	/* gtktextview has no 'activate' event, so we trap ENTER here */
-	case GDK_Return:
-	case GDK_KP_Enter:
-		if (!(evt->state & GDK_CONTROL_MASK))
-		{
-			g_signal_stop_emission_by_name (G_OBJECT (wid), "key_press_event");
-			mg_inputbox_cb (wid, sess->gui);
-		}
-#endif
 	}
 
 	return 0;
@@ -402,6 +388,7 @@ key_load_defaults ()
 		"A\n3\nChange Page\nD1:3\nD2!\n\n"\
 		"A\n2\nChange Page\nD1:2\nD2!\n\n"\
 		"A\n1\nChange Page\nD1:1\nD2!\n\n"\
+		"A\ngrave\nChange Page\nD1:auto\nD2!\n\n"\
 		"C\no\nInsert in Buffer\nD1:\nD2!\n\n"\
 		"C\nb\nInsert in Buffer\nD1:\nD2!\n\n"\
 		"C\nk\nInsert in Buffer\nD1:\nD2!\n\n"\
@@ -815,10 +802,6 @@ key_dialog_show ()
 	gtk_box_pack_end (GTK_BOX (vbox), hbox2, 0, 0, 1);
 
 	wid = gtk_xtext_new (colors, 0);
-	gtk_xtext_set_tint (GTK_XTEXT (wid), prefs.tint_red, prefs.tint_green, prefs.tint_blue);
-	gtk_xtext_set_background (GTK_XTEXT (wid),
-									  channelwin_pix,
-									  prefs.transparent);
 	gtk_widget_set_usize (wid, 0, 75);
 	gtk_box_pack_start (GTK_BOX (hbox2), wid, 1, 1, 1);
 	gtk_xtext_set_font (GTK_XTEXT (wid), prefs.font_normal);
@@ -964,10 +947,9 @@ key_load_kbs_helper_mod (char *in, int *out)
 static int
 key_load_kbs (char *filename)
 {
-	char *buf, *ibuf;
-	struct stat st;
+	char *buf, pbuf[256], ibuf[1024];
 	struct key_binding *kb = NULL, *last = NULL;
-	int fd, len, pnt = 0, state = 0, n;
+	int fd, len, state = 0, n;
 
 	if (filename == NULL)
 		fd = xchat_open_file ("keybindings.conf", O_RDONLY, 0, 0);
@@ -975,18 +957,16 @@ key_load_kbs (char *filename)
 		fd = xchat_open_file (filename, O_RDONLY, 0, XOF_FULLPATH);
 	if (fd < 0)
 		return 1;
-	if (fstat (fd, &st) != 0)
-		return 1;
-	ibuf = malloc (st.st_size);
-	read (fd, ibuf, st.st_size);
-	close (fd);
-
-	while (buf_get_line (ibuf, &buf, &pnt, st.st_size))
+	
+	while (waitline (fd, pbuf, sizeof(pbuf), FALSE) != -1)
 	{
+		buf = pbuf;
 		if (buf[0] == '#')
 			continue;
 		if (strlen (buf) == 0)
+		{
 			continue;
+		}
 
 		switch (state)
 		{
@@ -1013,13 +993,10 @@ key_load_kbs (char *filename)
 				/* Unknown keyname, abort */
 				if (last)
 					last->next = NULL;
-				free (ibuf);
-				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
 							 _("Unknown keyname %s in key bindings config file\nLoad aborted, please fix %s/keybindings.conf\n"),
 							 buf, get_xdir_utf8 ());
 				fe_message (ibuf, FE_MSG_ERROR);
-				free (ibuf);
 				return 2;
 			}
 			kb->keyname = gdk_keyval_name (n);
@@ -1051,13 +1028,10 @@ key_load_kbs (char *filename)
 			{
 				if (last)
 					last->next = NULL;
-				free (ibuf);
-				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
 							 _("Unknown action %s in key bindings config file\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
 				fe_message (ibuf, FE_MSG_ERROR);
-				free (ibuf);
 				return 3;
 			}
 			state = KBSTATE_DT1;
@@ -1072,13 +1046,10 @@ key_load_kbs (char *filename)
 
 			if (buf[0] != 'D')
 			{
-				free (ibuf);
-				ibuf = malloc (1024);
 				snprintf (ibuf, 1024,
 							 _("Expecting Data line (beginning Dx{:|!}) but got:\n%s\n\nLoad aborted, Please fix %s/keybindings\n"),
 							 buf, get_xdir_utf8 ());
 				fe_message (ibuf, FE_MSG_ERROR);
-				free (ibuf);
 				return 4;
 			}
 			switch (buf[1])
@@ -1135,12 +1106,13 @@ key_load_kbs (char *filename)
 			continue;
 		}
 	}
+	close(fd);
 	if (last)
 		last->next = NULL;
-	free (ibuf);
 	return 0;
 
  corrupt_file:
+	close(fd);
 	/*if (getenv ("XCHAT_DEBUG"))
 		abort ();*/
 	snprintf (ibuf, 1024,
@@ -1148,7 +1120,6 @@ key_load_kbs (char *filename)
 								 "Please fix %s/keybindings.conf\n"),
 						 get_xdir_utf8 ());
 	fe_message (ibuf, FE_MSG_ERROR);
-	free (ibuf);
 	return 5;
 }
 
@@ -1196,6 +1167,20 @@ key_action_handle_command (GtkWidget * wid, GdkEventKey * evt, char *d1,
 	return 0;
 }
 
+/*
+ * Check if the given session is inside the main window. This predicate
+ * is passed to lastact_pop as a way to filter out detached sessions.
+ * XXX: Consider moving this in a different file?
+ */
+static int
+session_check_is_tab(session *sess)
+{
+	if (!sess || !sess->gui)
+		return FALSE;
+
+	return (sess->gui->is_tab);
+}
+
 static int
 key_action_page_switch (GtkWidget * wid, GdkEventKey * evt, char *d1,
 								char *d2, struct session *sess)
@@ -1209,6 +1194,31 @@ key_action_page_switch (GtkWidget * wid, GdkEventKey * evt, char *d1,
 	if (!len)
 		return 1;
 
+	if (strcasecmp(d1, "auto") == 0)
+	{
+		session *newsess;
+
+		/* Auto switch makes no sense in detached sessions */
+		if (!sess->gui->is_tab)
+			return 1;
+
+		/* Obtain a session with recent activity */
+		newsess = lastact_getfirst(session_check_is_tab);
+		if (newsess)
+		{
+			/*
+			 * Only sessions in the current window should be considered (i.e.
+			 * we don't want to move the focus on a different window). This
+			 * call could, in theory, do this, but we checked before that
+			 * newsess->gui->is_tab and sess->gui->is_tab.
+			 */
+			mg_bring_tofront_sess(newsess);
+			return 0;
+		}
+		else
+			return 1;
+	}
+
 	for (i = 0; i < len; i++)
 	{
 		if (d1[i] < '0' || d1[i] > '9')
@@ -1454,7 +1464,7 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 			return 2;
 
 		cursor_pos = g_utf8_pointer_to_offset(text, ch);
-		if (cursor_pos && (g_utf8_get_char_validated(ch, -1) == ':' || 
+		if (cursor_pos && (g_utf8_get_char_validated(ch, -1) == ':' ||
 					g_utf8_get_char_validated(ch, -1) == ',' ||
 					g_utf8_get_char_validated(ch, -1) == prefs.nick_suffix[0]))
 		{
@@ -1465,7 +1475,7 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 	}
 
 	comp = skip_len;
-	
+
 	/* store the text following the cursor for reinsertion later */
 	if ((cursor_pos + skip_len) < len)
 		postfix = g_utf8_offset_to_pointer(text, cursor_pos + skip_len);
@@ -1484,14 +1494,14 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 		ent_start++;
 		is_cmd = 1;
 	}
-	
+
 	prefix_len = ent_start;
 	elen = cursor_pos - ent_start;
 
 	g_utf8_strncpy (ent, g_utf8_offset_to_pointer (text, prefix_len), elen);
 
 	is_nick = (ent[0] == '#' || ent[0] == '&' || is_cmd) ? 0 : 1;
-	
+
 	if (sess->type == SESS_DIALOG && is_nick)
 	{
 		/* tab in a dialog completes the other person's name */
@@ -1522,7 +1532,9 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 				{
 					tmp_list = g_list_prepend (tmp_list, xc_cmds[i].name);
 				}
+#if 0
 				tmp_list = plugin_command_list(tmp_list);
+#endif
 			}
 			else
 				tmp_list = chanlist_double_list (sess_list);
@@ -1540,13 +1552,13 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 			key_action_tab_clean ();
 			comp = 0;
 		}
-	
+
 #if GLIB_CHECK_VERSION(2,4,0)
 		list = g_completion_complete_utf8 (gcomp, comp ? old_gcomp.data : ent, &result);
 #else
 		list = g_completion_complete (gcomp, comp ? old_gcomp.data : ent, &result);
 #endif
-		
+
 		if (result == NULL) /* No matches found */
 		{
 			g_completion_free(gcomp);
@@ -1653,7 +1665,7 @@ key_action_tab_comp (GtkWidget *t, GdkEventKey *entry, char *d1, char *d2,
 			}
 		}
 	}
-	
+
 	if(result)
 	{
 		if (prefix_len)
