@@ -22,7 +22,7 @@
 #include <stdarg.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <unistd.h>
+#include "../common/stdinc.h"
 #include <fcntl.h>
 #include "fe-gtk.h"
 
@@ -211,7 +211,7 @@ gtkutil_file_req (const char *title, void *callback, void *userdata, char *filte
 	}
 	else
 	{
-		if (filter && (flags & FRF_FILTERISINITIAL))
+		if (filter && (flags & FRF_FILTERISINITIAL) && !(flags & FRF_WRITE)) 
 			gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (dialog), filter);
 	}
 
@@ -602,7 +602,7 @@ gtkutil_treeview_new (GtkWidget *box, GtkTreeModel *model,
 			case G_TYPE_STRING:	/* fall through */
 			default:
 				renderer = gtk_cell_renderer_text_new ();
-				attr = "text";
+				attr = "markup";
 				break;
 		}
 
