@@ -52,7 +52,6 @@
 #include "../common/xchatc.h"
 #include "../common/outbound.h"
 #include "../common/inbound.h"
-#include "../common/plugin.h"
 #include "../common/modes.h"
 #include "../common/url.h"
 #include "fe-gtk.h"
@@ -65,17 +64,14 @@
 #include "userlistgui.h"
 #include "chanview.h"
 #include "pixmaps.h"
-#include "plugin-tray.h"
+#include "tray.h"
 #include "xtext.h"
 #include "sync-menu.h"
 
-#ifdef USE_GTKSPELL
-#include <gtk/gtktextview.h>
-#include <gtkspell/gtkspell.h>
+#ifdef HAVE_LIBSEXY
+# include <libsexy/sexy.h>
 #endif
 
-#include "sexy-spell-entry.h"
-
 #define GUI_SPACING (3)
 #define GUI_BORDER (0)
 #define SCROLLBAR_SPACING (2)
@@ -93,10 +89,13 @@ enum
 };
 
 /* two different types of tabs */
-#define TAG_IRC 0		/* server, channel, dialog */
-#define TAG_UTIL 1	/* dcc, notify, chanlist */
+typedef enum {
+	TAG_IRC,
+	TAG_UTIL,
+} TagType;
 
 static void mg_create_entry (session *sess, GtkWidget *box);
+static void mg_create_search (session *sess, GtkWidget *box);
 static void mg_link_irctab (session *sess, int focus);
 
 static session_gui static_mg_gui;
@@ -115,59 +114,6 @@ static PangoAttrList *nickseen_list;
 static PangoAttrList *newmsg_list;
 static PangoAttrList *plain_list = NULL;
 
-
-#ifdef USE_GTKSPELL
-
-/* use these when it's a GtkTextView instead of GtkEntry */
-
-char *
-SPELL_ENTRY_GET_TEXT (GtkWidget *entry)
-{
-	static char *last = NULL;	/* warning: don't overlap 2 GET_TEXT calls! */
-	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
-	GtkTextIter start_iter, end_iter;
-
-	gtk_text_buffer_get_iter_at_offset (buf, &start_iter, 0);
-	gtk_text_buffer_get_end_iter (buf, &end_iter);
-	g_free (last);
-	last = gtk_text_buffer_get_text (buf, &start_iter, &end_iter, FALSE);
-	return last;
-}
-
-void
-SPELL_ENTRY_SET_POS (GtkWidget *entry, int pos)
-{
-	GtkTextIter iter;
-	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
-
-	gtk_text_buffer_get_iter_at_offset (buf, &iter, pos);
-	gtk_text_buffer_place_cursor (buf, &iter);
-}
-
-int
-SPELL_ENTRY_GET_POS (GtkWidget *entry)
-{
-	GtkTextIter cursor;
-	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
-
-	gtk_text_buffer_get_iter_at_mark (buf, &cursor, gtk_text_buffer_get_insert (buf));
-	return gtk_text_iter_get_offset (&cursor);
-}
-
-void
-SPELL_ENTRY_INSERT (GtkWidget *entry, const char *text, int len, int *pos)
-{
-	GtkTextIter iter;
-	GtkTextBuffer *buf = gtk_text_view_get_buffer (GTK_TEXT_VIEW (entry));
-
-	/* len is bytes. pos is chars. */
-	gtk_text_buffer_get_iter_at_offset (buf, &iter, *pos);
-	gtk_text_buffer_insert (buf, &iter, text, len);
-	*pos += g_utf8_strlen (text, len);
-}
-
-#endif
-
 static PangoAttrList *
 mg_attr_list_create (GdkColor *col, int size)
 {
@@ -218,7 +164,7 @@ mg_create_tab_colors (void)
 void
 fe_flash_window (session *sess)
 {
-	if (fe_gui_info (sess, 0) != 1)	/* only do it if not focused */
+	if (fe_gui_info(sess, 0) != 1)	/* only do it if not focused */
 		gtk_window_set_urgency_hint (GTK_WINDOW(sess->gui->window), TRUE);
 }
 
@@ -251,14 +197,14 @@ fe_set_tab_color (struct session *sess, int col)
 				server_sess->nick_said = FALSE;
 				chan_set_color (chan_get_parent (sess->res->tab), newdata_list);
 			}
-				
+
 			break;
 		case 2:	/* new message arrived in channel (light red) */
 			sess->new_data = FALSE;
 			sess->msg_said = TRUE;
 			sess->nick_said = FALSE;
 			chan_set_color (sess->res->tab, newmsg_list);
-			
+
 			if (chan_is_collapsed (sess->res->tab))
 			{
 				server_sess->new_data = FALSE;
@@ -266,7 +212,7 @@ fe_set_tab_color (struct session *sess, int col)
 				server_sess->nick_said = FALSE;
 				chan_set_color (chan_get_parent (sess->res->tab), newmsg_list);
 			}
-			
+
 			break;
 		case 3:	/* your nick has been seen (blue) */
 			sess->new_data = FALSE;
@@ -281,9 +227,10 @@ fe_set_tab_color (struct session *sess, int col)
 				server_sess->nick_said = TRUE;
 				chan_set_color (chan_get_parent (sess->res->tab), nickseen_list);
 			}
-				
+
 			break;
 		}
+		lastact_update(sess);
 	}
 }
 
@@ -376,21 +323,21 @@ fe_set_title (session *sess)
 	switch (type)
 	{
 	case SESS_DIALOG:
-		snprintf (tbuf, sizeof (tbuf), "conspire: %s %s @ %s",
-					 _("Dialog with"), sess->channel, server_get_network (sess->server, TRUE));
+		snprintf (tbuf, sizeof (tbuf), "%s@%s",
+					sess->channel, server_get_network (sess->server, TRUE));
 		break;
 	case SESS_SERVER:
-		snprintf (tbuf, sizeof (tbuf), "conspire: %s @ %s",
+		snprintf (tbuf, sizeof (tbuf), "%s@%s",
 					 sess->server->nick, server_get_network (sess->server, TRUE));
 		break;
 	case SESS_CHANNEL:
-		snprintf (tbuf, sizeof (tbuf), "conspire: %s @ %s / %s (%s) (%d users, %d ops, %d halfops, %d voices)",
-			 sess->server->nick, server_get_network (sess->server, TRUE),
-			 sess->channel, sess->current_modes ? sess->current_modes : "", sess->total, sess->ops, sess->hops, sess->voices);
+		snprintf (tbuf, sizeof (tbuf), "%s@%s (status: %s, %d users, %d ops, %d halfops, %d voices)",
+			 sess->channel, server_get_network (sess->server, TRUE),
+			 sess->current_modes ? sess->current_modes : "", sess->total, sess->ops, sess->hops, sess->voices);
 		break;
 	case SESS_NOTICES:
 	case SESS_SNOTICES:
-		snprintf (tbuf, sizeof (tbuf), "conspire: %s @ %s (notices)",
+		snprintf (tbuf, sizeof (tbuf), "%s@%s (notices)",
 					 sess->server->nick, server_get_network (sess->server, TRUE));
 		break;
 	default:
@@ -447,9 +394,6 @@ mg_configure_cb (GtkWidget *wid, GdkEventConfigure *event, session *sess)
 			gtk_window_get_size (GTK_WINDOW (wid), &prefs.dialog_width,
 										&prefs.dialog_height);
 		}
-
-		if (((GtkXText *) sess->gui->xtext)->transparent)
-			gtk_widget_queue_draw (sess->gui->xtext);
 	}
 
 	return FALSE;
@@ -506,6 +450,7 @@ mg_focus (session *sess)
 		sess->nick_said = FALSE;
 		sess->msg_said = FALSE;
 		sess->new_data = FALSE;
+		lastact_update(sess);
 		/* when called via mg_changui_new, is_tab might be true, but
 			sess->res->tab is still NULL. */
 		if (sess->res->tab)
@@ -716,7 +661,7 @@ mg_decide_userlist (session *sess, gboolean switch_to_current)
 	case SESS_DIALOG:
 		mg_userlist_showhide (sess, FALSE);	/* hide */
 		break;
-	default:		
+	default:
 		mg_userlist_showhide (sess, TRUE);	/* show */
 		mg_topicbar_showhide (sess, TRUE);
 	}
@@ -759,7 +704,6 @@ mg_populate (session *sess)
 	session_gui *gui = sess->gui;
 	restore_gui *res = sess->res;
 	int i, render = TRUE;
-	guint16 vis = gui->ul_hidden;
 
 	switch (sess->type)
 	{
@@ -798,11 +742,6 @@ mg_populate (session *sess)
 	if (gui->is_tab)
 		gtk_notebook_set_current_page (GTK_NOTEBOOK (gui->note_book), 0);
 
-	/* xtext size change? Then don't render, wait for the expose caused
-      by showing/hidding the userlist */
-	if (vis != gui->ul_hidden && gui->user_box->allocation.width > 1)
-		render = FALSE;
-
 	gtk_xtext_buffer_show (GTK_XTEXT (gui->xtext), res->buffer, render);
 	GTK_XTEXT (gui->xtext)->color_paste = sess->color_paste;
 
@@ -854,17 +793,8 @@ mg_populate (session *sess)
 			add_tip (sess->gui->throttlemeter->parent, res->queue_tip);
 	}
 
-	/* menu items */
-	GTK_CHECK_MENU_ITEM (gui->menu_item[MENU_ID_AWAY])->active = sess->server->is_away;
-	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_AWAY], sess->server->connected);
-	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_JOIN], sess->server->end_of_motd);
-	gtk_widget_set_sensitive (gui->menu_item[MENU_ID_DISCONNECT],
-									  sess->server->connected || sess->server->recondelay_tag);
-
 	mg_set_topic_tip (sess);
 
-	plugin_emit_dummy_print (sess, "Focus Tab");
-
 	/* recalculate line widths... window size may have changed on us. --nenolod */
 	gtk_xtext_set_time_stamp (sess->res->buffer, prefs.timestamp);
 	((xtext_buffer *)sess->res->buffer)->needs_recalc = TRUE;
@@ -1030,6 +960,12 @@ mg_beepmsg_cb (GtkCheckMenuItem *item, session *sess)
 		sess->beep = TRUE;
 }
 
+static void
+mg_away_cb (GtkCheckMenuItem *item, session *sess)
+{
+	handle_command(sess, "away", FALSE);
+}
+
 static void
 mg_hidejp_cb (GtkCheckMenuItem *item, session *sess)
 {
@@ -1058,156 +994,10 @@ mg_create_icon_item (char *label, char *stock, GtkWidget *menu,
 	gtk_widget_show (item);
 }
 
-static int
-mg_count_networks (void)
-{
-	int cons = 0;
-	GSList *list;
-
-	for (list = serv_list; list; list = list->next)
-	{
-		if (((server *)list->data)->connected)
-			cons++;
-	}
-	return cons;
-}
-
-static int
-mg_count_dccs (void)
-{
-	GSList *list;
-	struct DCC *dcc;
-	int dccs = 0;
-
-	list = dcc_list;
-	while (list)
-	{
-		dcc = list->data;
-		if ((dcc->type == TYPE_SEND || dcc->type == TYPE_RECV) &&
-			 dcc->dccstat == STAT_ACTIVE)
-			dccs++;
-		list = list->next;
-	}
-
-	return dccs;
-}
-
 void
-mg_open_quit_dialog (gboolean minimize_button)
+mg_quit (void)
 {
-	static GtkWidget *dialog = NULL;
-	GtkWidget *dialog_vbox1;
-	GtkWidget *table1;
-	GtkWidget *image;
-	GtkWidget *checkbutton1;
-	GtkWidget *label;
-	GtkWidget *dialog_action_area1;
-	GtkWidget *button;
-	char *text, *connecttext;
-	int cons;
-	int dccs;
-
-	if (dialog)
-	{
-		gtk_window_present (GTK_WINDOW (dialog));
-		return;
-	}
-
-	dccs = mg_count_dccs ();
-	cons = mg_count_networks ();
-	if (dccs + cons == 0 || !prefs.gui_quit_dialog)
-	{
-		xchat_exit ();
-		return;
-	}
-
-	dialog = gtk_dialog_new ();
-	gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Quit conspire?"));
-	gtk_window_set_transient_for (GTK_WINDOW (dialog), GTK_WINDOW (parent_window));
-	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
-	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-
-	dialog_vbox1 = GTK_DIALOG (dialog)->vbox;
-	gtk_widget_show (dialog_vbox1);
-
-	table1 = gtk_table_new (2, 2, FALSE);
-	gtk_widget_show (table1);
-	gtk_box_pack_start (GTK_BOX (dialog_vbox1), table1, TRUE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (table1), 6);
-	gtk_table_set_row_spacings (GTK_TABLE (table1), 12);
-	gtk_table_set_col_spacings (GTK_TABLE (table1), 12);
-
-	image = gtk_image_new_from_stock ("gtk-dialog-warning", GTK_ICON_SIZE_DIALOG);
-	gtk_widget_show (image);
-	gtk_table_attach (GTK_TABLE (table1), image, 0, 1, 0, 1,
-							(GtkAttachOptions) (GTK_FILL),
-							(GtkAttachOptions) (GTK_FILL), 0, 0);
-
-	checkbutton1 = gtk_check_button_new_with_mnemonic (_("Don't ask next time."));
-	gtk_widget_show (checkbutton1);
-	gtk_table_attach (GTK_TABLE (table1), checkbutton1, 0, 2, 1, 2,
-							(GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
-							(GtkAttachOptions) (0), 0, 4);
-
-	connecttext = g_strdup_printf (_("You are connected to %i IRC networks."), cons);
-	text = g_strdup_printf ("<span weight=\"bold\" size=\"larger\">%s</span>\n\n%s\n%s",
-								_("Are you sure you want to quit?"),
-								cons ? connecttext : "",
-								dccs ? _("Some file transfers are still active.") : "");
-	g_free (connecttext);
-	label = gtk_label_new (text);
-	g_free (text);
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table1), label, 1, 2, 0, 1,
-							(GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK | GTK_FILL),
-							(GtkAttachOptions) (GTK_EXPAND | GTK_SHRINK), 0, 0);
-	gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
-	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
-
-	dialog_action_area1 = GTK_DIALOG (dialog)->action_area;
-	gtk_widget_show (dialog_action_area1);
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1),
-										GTK_BUTTONBOX_END);
-
-	if (minimize_button)
-	{
-		button = gtk_button_new_with_mnemonic (_("_Minimize to Tray"));
-		gtk_widget_show (button);
-		gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 1);
-	}
-
-	button = gtk_button_new_from_stock ("gtk-cancel");
-	gtk_widget_show (button);
-	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
-											GTK_RESPONSE_CANCEL);
-	gtk_widget_grab_focus (button);
-
-	button = gtk_button_new_from_stock ("gtk-quit");
-	gtk_widget_show (button);
-	gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button, 0);
-
-	gtk_widget_show (dialog);
-
-	switch (gtk_dialog_run (GTK_DIALOG (dialog)))
-	{
-	case 0:
-		if (GTK_TOGGLE_BUTTON (checkbutton1)->active)
-			prefs.gui_quit_dialog = 0;
-		xchat_exit ();
-		break;
-	case 1: /* minimize to tray */
-		if (GTK_TOGGLE_BUTTON (checkbutton1)->active)
-		{
-			prefs.gui_tray_flags |= 1;
-			/*prefs.gui_quit_dialog = 0;*/
-		}
-		tray_toggle_visibility (TRUE);
-		break;
-	}
-
-	gtk_widget_destroy (dialog);
-	dialog = NULL;
+	xchat_exit();
 }
 
 void
@@ -1218,7 +1008,7 @@ mg_close_sess (session *sess)
 
 	if (sess_list->next == NULL)
 	{
-		mg_open_quit_dialog (FALSE);
+		mg_quit();
 		return;
 	}
 
@@ -1249,19 +1039,26 @@ mg_chan_remove (chan *ch)
 void
 mg_close_gen (chan *ch, GtkWidget *box)
 {
-	char *title = g_object_get_data (G_OBJECT (box), "title");
+	if (box != NULL)
+	{
+		char *title = g_object_get_data(G_OBJECT (box), "title");
 
-	if (title)
-		free (title);
-	if (!ch)
-		ch = g_object_get_data (G_OBJECT (box), "ch");
-	if (ch)
+		if (title != NULL)
+			free(title);
+		if (ch == NULL)
+			ch = g_object_get_data(G_OBJECT (box), "ch");
+	}
+
+	if (ch != NULL)
 	{
 		/* remove from notebook */
-		gtk_widget_destroy (box);
+		if (box != NULL)
+			gtk_widget_destroy(box);
+
 		/* remove the tab from chanview */
 		mg_chan_remove (ch);
-	} else
+	}
+	else if (box != NULL)
 	{
 		gtk_widget_destroy (gtk_widget_get_toplevel (box));
 	}
@@ -1303,6 +1100,18 @@ mg_link_gentab (chan *ch, GtkWidget *box)
 	g_object_unref (box);
 }
 
+static void
+mg_disconnect_cb (GtkWidget *item, session *sess)
+{
+	handle_command(sess, "disconnect", FALSE);
+}
+
+static void
+mg_reconnect_cb (GtkWidget *item, session *sess)
+{
+	handle_command(sess, "reconnect", FALSE);
+}
+
 static void
 mg_detach_tab_cb (GtkWidget *item, chan *ch)
 {
@@ -1422,7 +1231,6 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 {
 	GtkWidget *menu, *item;
 	session *sess = ud;
-	char buf[256];
 
 	/* shift-click to close a tab */
 	if ((event->state & GDK_SHIFT_MASK) && event->type == GDK_BUTTON_PRESS)
@@ -1438,18 +1246,17 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 
 	if (tag == TAG_IRC)
 	{
-		char *name = g_markup_escape_text (sess->channel[0] ? sess->channel : _("<none>"), -1);
-		snprintf (buf, sizeof (buf), "<span foreground=\"#3344cc\"><b>%s</b></span>", name);
-		g_free (name);
-		item = gtk_menu_item_new_with_label ("");
-		gtk_label_set_markup (GTK_LABEL (GTK_BIN (item)->child), buf);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
+		if (sess->type == SESS_SERVER)
+		{
+			mg_create_icon_item(_("_Disconnect"), GTK_STOCK_DISCONNECT, menu,
+				mg_disconnect_cb, sess);
+			mg_create_icon_item(_("_Reconnect"), GTK_STOCK_CONNECT, menu,
+				mg_reconnect_cb, sess);
 
-		/* separator */
-		item = gtk_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
+			item = gtk_menu_item_new ();
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+			gtk_widget_show (item);
+		}
 
 		menu_toggle_item (_("Beep on message"), menu, mg_beepmsg_cb, sess,
 								sess->beep);
@@ -1462,6 +1269,8 @@ mg_tab_contextmenu_cb (chanview *cv, chan *ch, int tag, gpointer ud, GdkEventBut
 		menu_toggle_item (_("Color paste"), menu, mg_colorpaste_cb, sess,
 								sess->color_paste);
 
+		menu_toggle_item (_("Marked Away"), menu, mg_away_cb, sess,
+								sess->server->is_away);
 	}
 
 	/* separator */
@@ -1498,7 +1307,7 @@ mg_dnd_drop_file (session *sess, char *target, char *uri)
 	while (*p)
 	{
 		next = strchr (p, '\r');
-		if (strncasecmp ("file:", p, 5) == 0)
+		if (g_ascii_strncasecmp ("file:", p, 5) == 0)
 		{
 			if (next)
 				*next = 0;
@@ -1583,6 +1392,7 @@ mg_userlist_button (GtkWidget * box, char *label, char *cmd,
 						  int a, int b, int c, int d)
 {
 	GtkWidget *wid = gtk_button_new_with_label (label);
+	gtk_button_set_relief(GTK_BUTTON(wid), GTK_RELIEF_NONE);
 	g_signal_connect (G_OBJECT (wid), "clicked",
 							G_CALLBACK (userlist_button_cb), cmd);
 	gtk_table_attach_defaults (GTK_TABLE (box), wid, a, b, c, d);
@@ -1815,7 +1625,7 @@ flagk_hit (GtkWidget * wid, struct session *sess)
 
 	if (serv->connected && sess->channel[0])
 	{
-		snprintf (modes, sizeof (modes), "-k %s", 
+		snprintf (modes, sizeof (modes), "-k %s",
 			  gtk_entry_get_text (GTK_ENTRY (sess->gui->key_entry)));
 
 		if (GTK_TOGGLE_BUTTON (wid)->active)
@@ -1904,7 +1714,7 @@ mg_limit_entry_cb (GtkWidget * igad, gpointer userdata)
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sess->gui->flag_l), FALSE);
 			return;
 		}
-		snprintf (modes, sizeof(modes), "+l %d", 
+		snprintf (modes, sizeof(modes), "+l %d",
 				atoi (gtk_entry_get_text (GTK_ENTRY (igad))));
 		serv->p_mode (serv, sess->channel, modes);
 		serv->p_join_info (serv, sess->channel);
@@ -2153,8 +1963,6 @@ mg_update_xtext (GtkWidget *wid)
 
 	gtk_xtext_set_palette (xtext, colors);
 	gtk_xtext_set_max_lines (xtext, prefs.max_lines);
-	gtk_xtext_set_tint (xtext, prefs.tint_red, prefs.tint_green, prefs.tint_blue);
-	gtk_xtext_set_background (xtext, channelwin_pix, prefs.transparent);
 	gtk_xtext_set_wordwrap (xtext, prefs.wordwrap);
 	gtk_xtext_set_show_marker (xtext, prefs.show_marker);
 	gtk_xtext_set_show_separator (xtext, prefs.indent_nicks ? prefs.show_separator : 0);
@@ -2165,7 +1973,7 @@ mg_update_xtext (GtkWidget *wid)
 		exit (1);
 	}
 
-	gtk_xtext_refresh (xtext, FALSE);
+	gtk_xtext_refresh (xtext);
 }
 
 /* handle errors reported by xtext */
@@ -2369,6 +2177,10 @@ mg_add_pane_signals (session_gui *gui)
 							G_CALLBACK (mg_rightpane_cb), gui);
 	g_signal_connect (G_OBJECT (gui->hpane_left), "notify::position",
 							G_CALLBACK (mg_leftpane_cb), gui);
+
+	/* XXX */
+	gtk_widget_hide(gui->shbox);
+
 	return FALSE;
 }
 
@@ -2411,6 +2223,7 @@ mg_create_center (session *sess, session_gui *gui, GtkWidget *box)
 	gtk_notebook_append_page (GTK_NOTEBOOK (book), vbox, NULL);
 	mg_create_topicbar (sess, vbox);
 	mg_create_textarea (sess, vbox);
+	mg_create_search (sess, vbox);
 	mg_create_entry (sess, vbox);
 
 	g_idle_add ((GSourceFunc)mg_add_pane_signals, gui);
@@ -2554,6 +2367,128 @@ mg_inputbox_rightclick (GtkEntry *entry, GtkWidget *menu)
 	mg_create_color_menu (menu, NULL);
 }
 
+static void
+search_toggle_match(GtkWidget *w, int state)
+{
+	if (state == 0)
+	{
+		/* No match */
+		GdkColor c;
+
+		gdk_color_parse ("red", &c);
+		gtk_widget_modify_bg(w, GTK_STATE_NORMAL, &c);
+		gtk_widget_modify_bg(w, GTK_STATE_SELECTED, &c);
+	}
+	else
+	{
+		/* Match */
+		gtk_widget_modify_bg(w, GTK_STATE_NORMAL, NULL);
+		gtk_widget_modify_bg(w, GTK_STATE_SELECTED, NULL);
+	}
+}
+
+#define SEARCH_CHANGE		1
+#define SEARCH_NEXT			2
+#define SEARCH_PREVIOUS		3
+static void
+search_handle_event(int search_type, session *sess)
+{
+	static textentry *last;
+	gchar *text;
+	gboolean backwards = FALSE;
+
+	switch (search_type)
+	{
+		case SEARCH_CHANGE:
+			/* On keypress, we want to re-search the whole buffer */
+			last = NULL;
+			break;
+		case SEARCH_NEXT:
+			break;
+		case SEARCH_PREVIOUS:
+			backwards = TRUE;
+			break;
+	}
+
+	text = gtk_editable_get_chars(GTK_EDITABLE(sess->gui->shentry), 0, -1);
+	last = gtk_xtext_search(GTK_XTEXT (sess->gui->xtext), text, last, FALSE, backwards);
+	if (!last)
+	{
+		search_toggle_match(sess->gui->shentry, 0);
+	}
+	else
+	{
+		search_toggle_match(sess->gui->shentry, 1);
+	}
+}
+
+static void
+search_handle_change(GtkEditable *editable, session *sess)
+{
+	search_handle_event(SEARCH_CHANGE, sess);
+}
+
+static void
+search_handle_previous(GtkWidget *button, session *sess)
+{
+	search_handle_event(SEARCH_PREVIOUS, sess);
+}
+
+static void
+search_handle_next(GtkWidget *button, session *sess)
+{
+	search_handle_event(SEARCH_NEXT, sess);
+}
+
+static void
+mg_create_search(session *sess, GtkWidget *box)
+{
+	GtkWidget *entry, *label, *next, *previous;
+	session_gui *gui = sess->gui;
+
+	gui->shbox = gtk_hbox_new(FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(box), gui->shbox, FALSE, FALSE, 0);
+
+	label = gtk_label_new(_("Find:"));
+	gtk_box_pack_start(GTK_BOX(gui->shbox), label, FALSE, FALSE, 0);
+
+	gui->shentry = entry = gtk_entry_new();
+	gtk_box_pack_start(GTK_BOX(gui->shbox), entry, TRUE, TRUE, 0);
+	gui->search_changed_signal = g_signal_connect(G_OBJECT(entry), "changed", G_CALLBACK(search_handle_change), sess);
+
+	previous = gtk_button_new_from_stock(GTK_STOCK_GO_BACK);
+	gtk_button_set_relief(GTK_BUTTON(previous), GTK_RELIEF_NONE);
+	gtk_box_pack_start(GTK_BOX(gui->shbox), previous, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(previous), "clicked", G_CALLBACK(search_handle_previous), sess);
+
+	next = gtk_button_new_from_stock(GTK_STOCK_GO_FORWARD);
+	gtk_button_set_relief(GTK_BUTTON(next), GTK_RELIEF_NONE);
+	gtk_box_pack_start(GTK_BOX(gui->shbox), next, FALSE, FALSE, 0);
+	g_signal_connect(G_OBJECT(next), "clicked", G_CALLBACK(search_handle_next), sess);
+}
+
+void
+mg_search_toggle(session *sess)
+{
+	if (GTK_WIDGET_VISIBLE(sess->gui->shbox))
+	{
+		gtk_widget_hide(sess->gui->shbox);
+		gtk_widget_grab_focus(sess->gui->input_box);
+		g_signal_handler_block(sess->gui->shentry, sess->gui->search_changed_signal);
+		gtk_entry_set_text(GTK_ENTRY(sess->gui->shentry), "");
+		g_signal_handler_unblock(sess->gui->shentry, sess->gui->search_changed_signal);
+	}
+	else
+	{
+		/* Reset search state */
+		search_toggle_match(sess->gui->shentry, 1);
+
+		/* Show and focus */
+		gtk_widget_show(sess->gui->shbox);
+		gtk_widget_grab_focus(sess->gui->shentry);
+	}
+}
+
 static void
 mg_create_entry (session *sess, GtkWidget *box)
 {
@@ -2563,8 +2498,12 @@ mg_create_entry (session *sess, GtkWidget *box)
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), hbox, 0, 0, 0);
 
+#ifdef HAVE_LIBSEXY
 	gui->input_box = entry = sexy_spell_entry_new ();
 	sexy_spell_entry_set_checked ((SexySpellEntry *)entry, prefs.gui_input_spell);
+#else
+	gui->input_box = entry = gtk_entry_new();
+#endif
 
 	gtk_entry_set_max_length (GTK_ENTRY (gui->input_box), 2048);
 	g_signal_connect (G_OBJECT (entry), "activate",
@@ -2632,7 +2571,7 @@ static void
 mg_create_tabs (session_gui *gui)
 {
 	gui->chanview = chanview_new (prefs.tab_layout, prefs.truncchans,
-											prefs.tab_sort, prefs.tab_icons,
+											prefs.tab_sort,
 											prefs.style_namelistgad ? input_style : NULL);
 	chanview_set_callbacks (gui->chanview, mg_switch_tab_cb, mg_xbutton_cb,
 									mg_tab_contextmenu_cb, (void *)mg_tabs_compare);
@@ -2646,11 +2585,12 @@ mg_tabwin_focus_cb (GtkWindow * win, GdkEventFocus *event, gpointer userdata)
 	if (current_sess)
 	{
 		gtk_xtext_check_marker_visibility (GTK_XTEXT (current_sess->gui->xtext));
-		plugin_emit_dummy_print (current_sess, "Focus Window");
 	}
 
 	gtk_window_set_urgency_hint(win, FALSE);
 
+	signal_emit("gui focused", 1, win);
+
 	return FALSE;
 }
 
@@ -2664,7 +2604,8 @@ mg_topwin_focus_cb (GtkWindow * win, GdkEventFocus *event, session *sess)
 
 	gtk_window_set_urgency_hint (win, FALSE);
 
-	plugin_emit_dummy_print (sess, "Focus Window");
+	signal_emit("gui focused", 1, win);
+
 	return FALSE;
 }
 
@@ -2797,7 +2738,7 @@ mg_tabwindow_de_cb (GtkWidget *widget, GdkEvent *event, gpointer user_data)
 		list = list->next;
 	}
 
-	mg_open_quit_dialog (TRUE);
+	mg_quit();
 	return TRUE;
 }
 
@@ -2896,7 +2837,7 @@ mg_add_generic_tab (char *name, char *title, void *family, GtkWidget *box)
 	gtk_notebook_append_page (GTK_NOTEBOOK (mg_gui->note_book), box, NULL);
 	gtk_widget_show (box);
 
-	ch = chanview_add (mg_gui->chanview, name, NULL, box, TRUE, TAG_UTIL, pix_util);
+	ch = chanview_add (mg_gui->chanview, name, family, box, TRUE, TAG_UTIL, pix_util);
 	chan_set_color (ch, plain_list);
 	/* FIXME: memory leak */
 	g_object_set_data (G_OBJECT (box), "title", strdup (title));
@@ -3030,10 +2971,7 @@ fe_set_away (server *serv)
 	{
 		sess = list->data;
 		if (sess->server == serv)
-		{
-			if (!sess->gui->is_tab || sess == current_tab)
-				GTK_CHECK_MENU_ITEM (sess->gui->menu_item[MENU_ID_AWAY])->active = serv->is_away;
-		}
+			break;
 		list = list->next;
 	}
 }
