@@ -1,4 +1,7 @@
-/* X-Chat
+/* Conspire
+ * Copyright (c) 2008 Robin Burchell
+ *
+ * X-Chat
  * Copyright (C) 2004 Peter Zelezny.
  */
 
@@ -343,52 +346,9 @@ servlist_move_network (ircnet *net, int delta)
 	}
 }
 
-#if 0
-static gboolean
-servlist_auto_find (GtkWidget *wid, GdkEventKey *evt, gpointer tree)
-{
-	GtkTreeModel *model = gtk_tree_view_get_model (tree);
-	GtkTreeIter iter;
-	unsigned char c;
-	unsigned char *net_name;
-
-	if (evt->keyval > 0x7a || evt->keyval < 0x41)
-		return FALSE;
-
-	c = toupper (evt->keyval);
-
-	/* scroll to a network that starts with the letter pressed */
-	if (gtk_tree_model_get_iter_first (model, &iter))
-	{
-		do
-		{
-			gtk_tree_model_get (model, &iter, 0, &net_name, -1);
-			if (net_name)
-			{
-				if (toupper (net_name[0]) == c)
-				{
-					servlist_select_and_show (tree, &iter, GTK_LIST_STORE (model));
-					g_free (net_name);
-					return TRUE;
-				}
-				g_free (net_name);
-			}
-		}
-		while (gtk_tree_model_iter_next (model, &iter));
-	}
-
-	return FALSE;
-}
-#endif
-
 static gboolean
 servlist_net_keypress_cb (GtkWidget *wid, GdkEventKey *evt, gpointer tree)
 {
-#if 0		/* GTK's auto-find does this better */
-	if (servlist_auto_find (wid, evt, tree))
-		return TRUE;
-#endif
-
 	if (!selected_net)
 		return FALSE;
 
@@ -478,7 +438,27 @@ servlist_update_from_entry (char **str, GtkWidget *entry)
 }
 
 static void
-servlist_edit_close_cb (GtkWidget *button, gpointer userdata)
+servlist_update_from_textview (char **str, GtkWidget *view)
+{
+	GtkTextIter start;
+	GtkTextIter end;
+	GtkTextBuffer *buffer;
+
+	if (*str)
+		free(*str);
+
+	buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+
+	/* Obtain iters for the start and end of points of the buffer */
+	gtk_text_buffer_get_start_iter(buffer, &start);
+	gtk_text_buffer_get_end_iter(buffer, &end);
+
+	/* Get the entire buffer text. */
+	*str = gtk_text_buffer_get_text(buffer, &start, &end, FALSE);
+}
+
+static void
+servlist_closegui(GtkWidget *wid, session *sess)
 {
 	ircnet *net = selected_net;
 
@@ -488,7 +468,7 @@ servlist_edit_close_cb (GtkWidget *button, gpointer userdata)
 	servlist_update_from_entry (&net->real, edit_entry_real);
 
 	servlist_update_from_entry (&net->autojoin, edit_entry_join);
-	servlist_update_from_entry (&net->command, edit_entry_cmd);
+	servlist_update_from_textview(&net->command, edit_entry_cmd);
 	servlist_update_from_entry (&net->nickserv, edit_entry_nickserv);
 	servlist_update_from_entry (&net->pass, edit_entry_pass);
 
@@ -499,13 +479,6 @@ servlist_edit_close_cb (GtkWidget *button, gpointer userdata)
 	edit_win = NULL;
 }
 
-static gint
-servlist_editwin_delete_cb (GtkWidget *win, GdkEventAny *event, gpointer none)
-{
-	servlist_edit_close_cb (NULL, NULL);
-	return FALSE;
-}
-
 static gboolean
 servlist_configure_cb (GtkWindow *win, GdkEventConfigure *event, gpointer none)
 {
@@ -520,13 +493,18 @@ servlist_edit_cb (GtkWidget *but, gpointer none)
 	if (!servlist_has_selection (GTK_TREE_VIEW (networks_tree)))
 		return;
 
+	if (edit_win)
+	{
+		/* Disallow multiple. */
+		fe_message (_("You may only edit one network at a time."), FE_MSG_ERROR);
+		return;
+	}
+
 	edit_win = servlist_open_edit (serverlist_win, selected_net);
 	gtkutil_set_icon (edit_win);
 	servlist_servers_populate (selected_net, edit_tree);
 	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (edit_tree))),
 							"changed", G_CALLBACK (servlist_server_row_cb), NULL);
-	g_signal_connect (G_OBJECT (edit_win), "delete_event",
-						 	G_CALLBACK (servlist_editwin_delete_cb), 0);
 	g_signal_connect (G_OBJECT (edit_tree), "key_press_event",
 							G_CALLBACK (servlist_serv_keypress_cb), 0);
 	gtk_widget_show (edit_win);
@@ -816,6 +794,54 @@ servlist_create_entry (GtkWidget *table, char *labeltext, int row,
 	return entry;
 }
 
+static GtkWidget *
+servlist_create_textview (GtkWidget *table, char *labeltext, int row,
+							  char *def, GtkWidget **label_ret, char *tip)
+{
+	GtkWidget *label, *entry, *scrolledwindow;
+	GtkTextBuffer *buffer;
+
+
+
+	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (scrolledwindow);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
+											  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
+													 GTK_SHADOW_IN);
+
+
+	label = gtk_label_new_with_mnemonic (labeltext);
+	if (label_ret)
+		*label_ret = label;
+	gtk_widget_show (label);
+
+	gtk_table_attach (GTK_TABLE (table), label, 1, 2, row, row+1,
+							GTK_FILL, 0, 0, 0);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
+
+	entry = gtk_text_view_new ();
+	gtk_container_add (GTK_CONTAINER (scrolledwindow), entry);
+	add_tip(entry, tip);
+	gtk_widget_show (entry);
+
+	/* Set text. */
+	if (def)
+	{
+		/* Obtain the buffer associated with the widget. */
+		buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(entry));
+		/* Set the default buffer text. */ 
+		gtk_text_buffer_set_text (buffer, def, -1);
+	}
+
+	gtk_table_attach (GTK_TABLE (table), scrolledwindow, 2, 3, row, row+1,
+							GTK_FILL|GTK_EXPAND, 0, 0, 0);
+	gtk_label_set_mnemonic_widget (GTK_LABEL (label), entry);
+
+	return entry;
+}
+
+
 static gint
 servlist_delete_cb (GtkWidget *win, GdkEventAny *event, gpointer userdata)
 {
@@ -950,6 +976,7 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 	GtkWidget *label17;
 	GtkWidget *label16;
 	GtkWidget *label21;
+	GtkWidget *label22;
 	GtkWidget *label34;
 	GtkWidget *comboboxentry_charset;
 	GtkWidget *hbox1;
@@ -959,9 +986,6 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 	GtkWidget *buttonadd;
 	GtkWidget *buttonremove;
 	GtkWidget *buttonedit;
-	GtkWidget *hseparator2;
-	GtkWidget *hbuttonbox4;
-	GtkWidget *button10;
 	GtkWidget *check;
 	GtkTreeModel *model;
 	GtkListStore *store;
@@ -969,20 +993,9 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 	char buf[128];
 	char buf2[128 + 8];
 
-	editwindow = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-	gtk_container_set_border_width (GTK_CONTAINER (editwindow), 4);
-	snprintf (buf, sizeof (buf), _("conspire: Edit %s"), net->name);
-	gtk_window_set_title (GTK_WINDOW (editwindow), buf);
-	gtk_window_set_default_size (GTK_WINDOW (editwindow), 354, 0);
-	gtk_window_set_position (GTK_WINDOW (editwindow), GTK_WIN_POS_MOUSE);
-	gtk_window_set_transient_for (GTK_WINDOW (editwindow), GTK_WINDOW (parent));
-	gtk_window_set_modal (GTK_WINDOW (editwindow), TRUE);
-	gtk_window_set_type_hint (GTK_WINDOW (editwindow), GDK_WINDOW_TYPE_HINT_DIALOG);
-	gtk_window_set_role (GTK_WINDOW (editwindow), "editserv");
-
-	vbox5 = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (vbox5);
-	gtk_container_add (GTK_CONTAINER (editwindow), vbox5);
+	char tbuf[256];
+	snprintf (tbuf, sizeof tbuf, _("conspire: Network Editor (%s)"), net->name);
+	editwindow = mg_create_generic_tab(_("Network Editor"), tbuf, FALSE, TRUE, servlist_closegui, NULL, 550, 200, &vbox5, NULL);
 
 	table3 = gtk_table_new (17, 3, FALSE);
 	gtk_widget_show (table3);
@@ -1043,11 +1056,6 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 									  net->autojoin, 0,
 				  _("Channels to join, separated by commas, but not spaces!"));
 
-	edit_entry_cmd =
-		servlist_create_entry (table3, _("Connect command:"), 16,
-									  net->command, 0,
-					_("Extra command to execute after connecting. If you need more than one, set this to LOAD -e <filename>, where <filename> is a text-file full of commands to execute."));
-
 	edit_entry_nickserv =
 		servlist_create_entry (table3, _("Nickserv password:"), 17,
 									  net->nickserv, 0,
@@ -1061,12 +1069,12 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 	gtk_entry_set_visibility (GTK_ENTRY (edit_entry_pass), FALSE);
 
 	edit_entry_sasl_user =
-		servlist_create_entry (table3, _("SASL account username:"), 19,
+		servlist_create_entry (table3, _("SASL username:"), 19,
 									  net->sasl_user, 0,
 					_("If your network uses SASL, put your username here."));
 
 	edit_entry_sasl_pass =
-		servlist_create_entry (table3, _("SASL account password:"), 20,
+		servlist_create_entry (table3, _("SASL password:"), 20,
 									  net->sasl_pass, 0,
 					_("If your network uses SASL, put your password here."));
 	gtk_entry_set_visibility (GTK_ENTRY (edit_entry_sasl_pass), FALSE);
@@ -1087,6 +1095,22 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 							(GtkAttachOptions) (GTK_FILL),
 							(GtkAttachOptions) (GTK_FILL), 0, 0);
 
+
+	label22 = bold_label (_("Perform"));
+	gtk_table_attach (GTK_TABLE (table3), label22, 0, 3, 22, 23,
+							(GtkAttachOptions) (GTK_FILL),
+							(GtkAttachOptions) (0), 0, 3);
+
+
+	edit_entry_cmd =
+		servlist_create_textview (table3, _("Connect commands:"), 24,
+									  net->command, 0,
+					_("Extra commands to execute after connecting. Alternatively, to run a large series of commands on connect, use LOAD -e <filename>, where <filename> is a text-file full of commands to execute."));
+
+
+
+	/*********** FINITO *************/
+
 	hbox1 = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox1);
 	gtk_table_attach (GTK_TABLE (table3), hbox1, 1, 3, 1, 2,
@@ -1148,23 +1172,6 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 	gtk_container_add (GTK_CONTAINER (vbuttonbox1), buttonedit);
 	GTK_WIDGET_SET_FLAGS (buttonedit, GTK_CAN_DEFAULT);
 
-	hseparator2 = gtk_hseparator_new ();
-	gtk_widget_show (hseparator2);
-	gtk_box_pack_start (GTK_BOX (vbox5), hseparator2, FALSE, FALSE, 8);
-
-	hbuttonbox4 = gtk_hbutton_box_new ();
-	gtk_widget_show (hbuttonbox4);
-	gtk_box_pack_start (GTK_BOX (vbox5), hbuttonbox4, FALSE, FALSE, 0);
-	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4),
-										GTK_BUTTONBOX_END);
-
-	button10 = gtk_button_new_from_stock ("gtk-close");
-	g_signal_connect (G_OBJECT (button10), "clicked",
-							G_CALLBACK (servlist_edit_close_cb), 0);
-	gtk_widget_show (button10);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox4), button10);
-	GTK_WIDGET_SET_FLAGS (button10, GTK_CAN_DEFAULT);
-
 	if (net->flags & FLAG_USE_GLOBAL)
 	{
 		gtk_widget_hide (edit_label_nick);
@@ -1180,9 +1187,6 @@ servlist_open_edit (GtkWidget *parent, ircnet *net)
 		gtk_widget_hide (edit_entry_real);
 	}
 
-	gtk_widget_grab_focus (button10);
-	gtk_widget_grab_default (button10);
-
 	return editwindow;
 }
 
@@ -1214,10 +1218,7 @@ servlist_open_networks (void)
 	GtkWidget *button_remove;
 	GtkWidget *button_edit;
 	GtkWidget *button_sort;
-	GtkWidget *hseparator1;
-	GtkWidget *hbuttonbox1;
 	GtkWidget *button_connect;
-	GtkWidget *button_close;
 	GtkTreeModel *model;
 	GtkListStore *store;
 	GtkCellRenderer *renderer;
@@ -1399,23 +1400,7 @@ servlist_open_networks (void)
 	gtk_container_add (GTK_CONTAINER (vbuttonbox2), button_sort);
 	GTK_WIDGET_SET_FLAGS (button_sort, GTK_CAN_DEFAULT);
 
-	hseparator1 = gtk_hseparator_new ();
-	gtk_widget_show (hseparator1);
-	gtk_box_pack_start (GTK_BOX (vbox1), hseparator1, FALSE, TRUE, 4);
-
-	hbuttonbox1 = gtk_hbutton_box_new ();
-	gtk_widget_show (hbuttonbox1);
-	gtk_box_pack_start (GTK_BOX (vbox1), hbuttonbox1, FALSE, TRUE, 0);
-	gtk_container_set_border_width (GTK_CONTAINER (hbuttonbox1), 8);
-
-	button_close = gtk_button_new_from_stock ("gtk-close");
-	gtk_widget_show (button_close);
-	g_signal_connect (G_OBJECT (button_close), "clicked",
-							G_CALLBACK (servlist_close_cb), 0);
-	gtk_container_add (GTK_CONTAINER (hbuttonbox1), button_close);
-	GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
-
-	button_connect = gtkutil_button (hbuttonbox1, GTK_STOCK_CONNECT, NULL,
+	button_connect = gtkutil_button (vbuttonbox2, GTK_STOCK_CONNECT, NULL,
 												servlist_connect_cb, NULL, _("C_onnect"));
 	GTK_WIDGET_SET_FLAGS (button_connect, GTK_CAN_DEFAULT);
 
@@ -1424,7 +1409,6 @@ servlist_open_networks (void)
 	gtk_label_set_mnemonic_widget (GTK_LABEL (label7), entry5);
 
 	gtk_widget_grab_focus (networks_tree);
-	gtk_widget_grab_default (button_close);
 	return servlist;
 }
 
@@ -1433,6 +1417,13 @@ fe_serverlist_open (session *sess)
 {
 	servlist_sess = sess;
 
+	/* fix: don't allow multiple serverlist editors, causes issues (crash at worst) */
+	if (serverlist_win)
+	{
+		fe_message (_("Server Editor is already open."), FE_MSG_ERROR);
+		return;
+	}
+
 	serverlist_win = servlist_open_networks ();
 	gtkutil_set_icon (serverlist_win);
 
