@@ -21,7 +21,7 @@
 #include "palette.h"
 #include "pixmaps.h"
 #include "menu.h"
-#include "plugin-tray.h"
+#include "tray.h"
 
 #include <gtk/gtkcolorseldialog.h>
 #include <gtk/gtktable.h>
@@ -48,11 +48,10 @@
 #include <gtk/gtktreeselection.h>
 #include <gtk/gtkcellrenderertext.h>
 #include <gtk/gtkhscale.h>
-#ifdef USE_GTKSPELL
-#include <gtk/gtktextview.h>
-#include <gtkspell/gtkspell.h>
+
+#ifdef HAVE_LIBSEXY
+# include <libsexy/sexy.h>
 #endif
-#include "sexy-spell-entry.h"
 
 GtkStyle *create_input_style (GtkStyle *);
 
@@ -60,13 +59,11 @@ GtkStyle *create_input_style (GtkStyle *);
 
 static int last_selected_page = 0;
 static gboolean color_change;
-static GtkWidget *cancel_button;
 
 enum
 {
 	ST_END,
 	ST_TOGGLE,
-	ST_TOGGLR,
 	ST_3OGGLE,
 	ST_ENTRY,
 	ST_EFONT,
@@ -100,11 +97,15 @@ static const setting textbox_settings[] =
 	{ST_NUMBER,	N_("Scrollback lines:"), &prefs.max_lines,0,0,100000},
 	{ST_TOGGLE, N_("Colored nick names"), &prefs.colorednicks,
 					N_("Give each person on IRC a different color"),0,0},
-	{ST_TOGGLR, N_("Indent nick names"), &prefs.indent_nicks,
+	{ST_TOGGLE, N_("Colored nicks (hilighted messages)"), &prefs.coloredhnicks,
+		N_("Give each person on IRC a different color for hilighted messages"), 0, 0},
+	{ST_TOGGLE, N_("Indent nick names"), &prefs.indent_nicks,
 					N_("Make nick names right-justified"),0,0},
 	{ST_TOGGLE, N_("Show marker line"), &prefs.show_marker,
 					N_("Insert a red line after the last read text."),0,0},
-	{ST_TOGGLR, N_("Automatically replay logged chats"), &prefs.text_replay,
+	{ST_TOGGLE, N_("Show redundant nicks"), &prefs.redundant_nickstamps,
+		N_("Show nicks when users enter multiple lines of text in a row"), 0, 0},
+	{ST_TOGGLE, N_("Automatically replay logged chats"), &prefs.text_replay,
 					N_("Automatically replays logs for channels and queries."),0,0},
 
 	{ST_HEADER,	N_("Time Stamps"),0,0,0},
@@ -115,7 +116,7 @@ static const setting textbox_settings[] =
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
-static const char *const tabcompmenu[] = 
+static const char *const tabcompmenu[] =
 {
 	N_("A-Z"),
 	N_("Last-spoke order"),
@@ -127,6 +128,9 @@ static const setting inputbox_settings[] =
 	{ST_HEADER, N_("Input box"),0,0,0},
 	{ST_TOGGLE, N_("Use the Text box font and colors"), &prefs.style_inputbox,0,0,0},
 	{ST_TOGGLE, N_("Spell checking"), &prefs.gui_input_spell,0,0,0},
+#ifdef REGEX_SUBSTITUTION
+	{ST_TOGGLE, N_("Enable regex substitution"), &prefs.text_regex_replace, N_("Use regular expressions on outbound text"), 0, 0},
+#endif
 
 	{ST_HEADER, N_("Nick Completion"),0,0,0},
 	{ST_TOGGLE, N_("Automatic nick completion (without TAB key)"), &prefs.nickcompletion,
@@ -135,25 +139,29 @@ static const setting inputbox_settings[] =
 	{ST_MENU,	N_("Nick completion sorted:"), &prefs.completion_sort, 0, tabcompmenu, 0},
 	{ST_NUMBER,	N_("Nick completion minimum length:"), &prefs.completion_amount, 0, 0,30},
 
-#if 0	/* obsolete */
 	{ST_HEADER, N_("Input Box Codes"),0,0,0},
 	{ST_TOGGLE, N_("Interpret %nnn as an ASCII value"), &prefs.perc_ascii,0,0,0},
 	{ST_TOGGLE, N_("Interpret %C, %B as Color, Bold etc"), &prefs.perc_color,0,0,0},
-#endif
+
+	{ST_HEADER, N_("Text Overflow"),0,0,0},
+	{ST_ENTRY,  N_("Stop indicator"), &prefs.text_overflow_stop,
+		N_("Used to indicate the end of any message due to overflow."), 0},
+	{ST_NUMBER, N_("Overflow length"), &prefs.text_overflow_limit,
+		N_("Used to determine the length at which to handle overflow."), 0},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
-/*static const char *const lagmenutext[] = 
+static const char *const lagmenutext[] =
 {
 	N_("Off"),
 	N_("Graph"),
 	N_("Info text"),
 	N_("Both"),
 	NULL
-};*/
+};
 
-static const char *const ulmenutext[] = 
+static const char *const ulmenutext[] =
 {
 	N_("A-Z, Ops first"),
 	N_("A-Z"),
@@ -200,9 +208,9 @@ static const setting userlist_settings[] =
 	{ST_HEADER,	N_("Action Upon Double Click"),0,0,0},
 	{ST_ENTRY,	N_("Execute command:"), &prefs.doubleclickuser, 0, 0},
 
-/*	{ST_HEADER,	N_("Extra Gadgets"),0,0,0},
+	{ST_HEADER,	N_("Extra Gadgets"),0,0,0},
 	{ST_MENU,	N_("Lag meter:"), &prefs.lagometer, 0, lagmenutext, 0},
-	{ST_MENU,	N_("Throttle meter:"), &prefs.throttlemeter, 0, lagmenutext, 0},*/
+	{ST_MENU,	N_("Throttle meter:"), &prefs.throttlemeter, 0, lagmenutext, 0},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
@@ -262,17 +270,22 @@ static const setting filexfer_settings[] =
 	{ST_EFOLDER,N_("Move completed files to:"), &prefs.dcc_completed_dir, 0, 0},
 	{ST_TOGGLE, N_("Save nick name in filenames"), &prefs.dccwithnick, 0, 0, 0},
 
+	{ST_HEADER,	N_("Auto Open DCC Windows"),0,0,0},
+	{ST_TOGGLE, N_("Send window"), &prefs.autoopendccsendwindow, 0, 0, 0},
+	{ST_TOGGLE, N_("Receive window"), &prefs.autoopendccrecvwindow, 0, 0, 0},
+	{ST_TOGGLE, N_("Chat window"), &prefs.autoopendccchatwindow, 0, 0, 0},
+
 	{ST_HEADER, N_("Network Settings"), 0, 0, 0},
 	{ST_TOGGLE, N_("Get my address from the IRC server"), &prefs.ip_from_server,
 					N_("Asks the IRC server for your real address. Use this if you have a 192.168.*.* address!"), 0, 0},
 	{ST_ENTRY,	N_("DCC IP address:"), &prefs.dcc_ip_str,
 					N_("Claim you are at this address when offering files."), 0},
 	{ST_NUMBER,	N_("First DCC send port:"), &prefs.first_dcc_send_port, 0, 0, 65535},
-	{ST_NUMBER,	N_("Last DCC send port:"), &prefs.last_dcc_send_port, 0, 
+	{ST_NUMBER,	N_("Last DCC send port:"), &prefs.last_dcc_send_port, 0,
 		(const char **)N_("!Leave ports at zero for full range."), 65535},
 
 	{ST_HEADER, N_("Maximum File Transfer Speeds (bytes per second)"), 0, 0, 0},
-	{ST_NUMBER,	N_("One upload:"), &prefs.dcc_max_send_cps, 
+	{ST_NUMBER,	N_("One upload:"), &prefs.dcc_max_send_cps,
 					N_("Maximum speed for one transfer"), 0, 1000000},
 	{ST_NUMBER,	N_("One download:"), &prefs.dcc_max_get_cps,
 					N_("Maximum speed for one transfer"), 0, 1000000},
@@ -319,6 +332,8 @@ static const setting alert_settings[] =
 	{ST_HEADER,	N_("Highlighted Messages"),0,0,0},
 	{ST_LABEL,	N_("Highlighted messages are ones where your nickname is mentioned, but also:"), 0, 0, 0, 1},
 
+        {ST_TOGGLE,	N_("Enable highlighting:"), &prefs.hilight_enable, 0, 0},
+
 	{ST_ENTRY,	N_("Extra words to highlight:"), &prefs.irc_extra_hilight, 0, 0},
 	{ST_ENTRY,	N_("Nick names not to highlight:"), &prefs.irc_no_hilight, 0, 0},
 	{ST_ENTRY,	N_("Nick names to always highlight:"), &prefs.irc_nick_hilight, 0, 0},
@@ -342,27 +357,18 @@ static const setting general_settings[] =
 	{ST_HEADER,	N_("Other Features"), 0, 0, 0},
 	{ST_TOGGLE,	N_("Strip IRCd quit messages"), &prefs.strip_quits,
 					N_("Strip quit prefixes like \"Quit:\" on IRC systems which use them."), 0, 0},
-	{ST_TOGGLE,	N_("Display redundant nickstamps"), &prefs.redundant_nickstamps,
-					N_("Displays the nickname associated with every line. Disabling this creates xchat-gnome like behaviour."), 0, 0},
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
-#if 0
 static const setting advanced_settings[] =
 {
 	{ST_HEADER,	N_("Advanced Settings"),0,0,0},
 	{ST_NUMBER,	N_("Auto reconnect delay:"), &prefs.recon_delay, 0, 0, 9999},
-	{ST_TOGGLE,	N_("Display MODEs in raw form"), &prefs.raw_modes, 0, 0, 0},
 	{ST_TOGGLE,	N_("Whois on notify"), &prefs.whois_on_notifyonline, N_("Sends a /WHOIS when a user comes online in your notify list"), 0, 0},
 	{ST_TOGGLE,	N_("Hide join and part messages"), &prefs.confmode, N_("Hide channel join/part messages by default"), 0, 0},
-	{ST_HEADER,	N_("Auto Open DCC Windows"),0,0,0},
-	{ST_TOGGLE, N_("Send window"), &prefs.autoopendccsendwindow, 0, 0, 0},
-	{ST_TOGGLE, N_("Receive window"), &prefs.autoopendccrecvwindow, 0, 0, 0},
-	{ST_TOGGLE, N_("Chat window"), &prefs.autoopendccchatwindow, 0, 0, 0},
 
 	{ST_END, 0, 0, 0, 0, 0}
 };
-#endif
 
 static const setting logging_settings[] =
 {
@@ -424,8 +430,14 @@ static const setting network_settings[] =
 	{ST_END, 0, 0, 0, 0, 0}
 };
 
+static const setting gui_colors_gtk[] =
+{
+	{ST_TOGGLE, N_("Use colors derived from GTK theme"), &prefs.gtk_colors, 0, 0, 0},
+	{ST_END, 0, 0, 0, 0, 0}
+};
+
 #define setup_get_int(pr,set) *(((int *)pr)+set->offset)
-#define setup_get_int3(pr,off) *(((int *)pr)+off) 
+#define setup_get_int3(pr,off) *(((int *)pr)+off)
 
 #define setup_set_int(pr,set,num) *((int *)pr+set->offset)=num
 
@@ -510,23 +522,8 @@ setup_toggle_cb (GtkToggleButton *but, const setting *set)
 	}
 }
 
-static void
-setup_create_toggleR (GtkWidget *tab, int row, const setting *set)
-{
-	GtkWidget *wid;
-
-	wid = gtk_check_button_new_with_label (_(set->label));
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid), *((int *) set->ptr));
-	g_signal_connect (G_OBJECT (wid), "toggled",
-							G_CALLBACK (setup_toggle_cb), (gpointer)set);
-	if (set->tooltip)
-		add_tip (wid, _(set->tooltip));
-	gtk_table_attach (GTK_TABLE (tab), wid, 4, 5, row, row + 1,
-							GTK_EXPAND | GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, 0, 0);
-}
-
 static GtkWidget *
-setup_create_toggleL (GtkWidget *tab, int row, const setting *set)
+setup_create_toggle (GtkWidget *tab, int row, const setting *set)
 {
 	GtkWidget *wid;
 
@@ -906,7 +903,7 @@ setup_create_entry (GtkWidget *table, int row, const setting *set)
 	if (set->ptr == &prefs.proxy_user)
 		proxy_user = wid;
 	if (set->ptr == &prefs.proxy_pass)
-		proxy_pass = wid; 
+		proxy_pass = wid;
 
 	/* only http and Socks5 can auth */
 	if ( (set->ptr == &prefs.proxy_pass ||
@@ -1010,12 +1007,8 @@ setup_create_page (const setting *set)
 		case ST_EFOLDER:
 			wid = setup_create_entry (tab, row, &set[i]);
 			break;
-		case ST_TOGGLR:
-			row--;
-			setup_create_toggleR (tab, row, &set[i]);
-			break;
 		case ST_TOGGLE:
-			wid = setup_create_toggleL (tab, row, &set[i]);
+			wid = setup_create_toggle (tab, row, &set[i]);
 			do_disable = set[i].extra;
 			break;
 		case ST_3OGGLE:
@@ -1180,7 +1173,7 @@ setup_create_other_color (char *text, int num, int row, GtkWidget *tab)
 static GtkWidget *
 setup_create_color_page (void)
 {
-	GtkWidget *tab, *box, *label;
+	GtkWidget *tab, *box, *label, *wid;
 	int i;
 
 	box = gtk_vbox_new (FALSE, 0);
@@ -1226,6 +1219,15 @@ setup_create_color_page (void)
 	setup_create_other_colorR (_("Away user:"), COL_AWAY, 10, tab);
 	setup_create_other_color (_("Highlight:"), COL_HILIGHT, 11, tab);
 
+	setup_create_header(tab, 15, N_("Color options:"));
+
+	wid = gtk_check_button_new_with_label (_(gui_colors_gtk[0].label));
+	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (wid), *((int *) gui_colors_gtk[0].ptr));
+	g_signal_connect (G_OBJECT (wid), "toggled",
+							G_CALLBACK (setup_toggle_cb), (gpointer)&gui_colors_gtk[0]);
+	gtk_table_attach (GTK_TABLE (tab), wid, 2, 9, 16, 16 + 1,
+							GTK_SHRINK | GTK_FILL, GTK_SHRINK | GTK_FILL, LABEL_INDENT, 0);
+
 	return box;
 }
 
@@ -1273,7 +1275,7 @@ static const char *const cata[] =
 		N_("Alerts"),
 		N_("General"),
 		N_("Logging"),
-/*		N_("Advanced"),*/
+		N_("Advanced"),
 		NULL,
 	N_("Network"),
 		N_("Network setup"),
@@ -1297,6 +1299,7 @@ setup_create_pages (GtkWidget *box)
 	setup_add_page (cata[8], book, setup_create_page (alert_settings));
 	setup_add_page (cata[9], book, setup_create_page (general_settings));
 	setup_add_page (cata[10], book, setup_create_page (logging_settings));
+	setup_add_page (cata[11], book, setup_create_page(advanced_settings));
 	setup_add_page (cata[14], book, setup_create_page (network_settings));
 	setup_add_page (cata[15], book, setup_create_page (filexfer_settings));
 
@@ -1410,10 +1413,6 @@ setup_apply_entry_style (GtkWidget *entry)
 static void
 setup_apply_to_sess (session_gui *gui)
 {
-#ifdef USE_GTKSPELL
-	GtkSpell *spell;
-#endif
-
 	mg_update_xtext (gui->xtext);
 
 	if (prefs.style_namelistgad)
@@ -1440,21 +1439,9 @@ setup_apply_to_sess (session_gui *gui)
 	else
 		gtk_widget_hide (gui->button_box);
 
-#ifdef USE_GTKSPELL
-	spell = gtkspell_get_from_text_view (GTK_TEXT_VIEW (gui->input_box));
-	if (prefs.gui_input_spell)
-	{
-		if (!spell)
-			gtkspell_new_attach (GTK_TEXT_VIEW (gui->input_box), NULL, NULL);
-	}
-	else
-	{
-		if (spell)
-			gtkspell_detach (spell);
-	}
-#endif
-
+#ifdef HAVE_LIBSEXY
 	sexy_spell_entry_set_checked ((SexySpellEntry *)gui->input_box, prefs.gui_input_spell);
+#endif
 }
 
 static void
@@ -1531,7 +1518,8 @@ setup_apply (struct xchatprefs *pr)
 	int do_ulist = FALSE;
 	int do_layout = FALSE;
 
-	if (strcmp (pr->background, prefs.background) != 0)
+	if ((!pr->background && prefs.background) || (pr->background && !prefs.background) ||
+		(pr->background && prefs.background && strcmp(pr->background, prefs.background) != 0))
 		new_pix = TRUE;
 
 #define DIFF(a) (pr->a != prefs.a)
@@ -1599,13 +1587,22 @@ setup_ok_cb (GtkWidget *but, GtkWidget *win)
 static GtkWidget *
 setup_window_open (void)
 {
-	GtkWidget *wid, *win, *vbox, *hbox, *hbbox;
+	GtkWidget *win, *vbox, *hbox, *scrolledwindow;
 
-	win = gtkutil_window_new (_("conspire: Preferences"), "prefs", 0, 0, 3);
+	win = mg_create_generic_tab(_("Preferences"), _("Preferences"), FALSE, TRUE, setup_ok_cb, NULL, 0, 0, &vbox, NULL);
+
+	scrolledwindow = gtk_scrolled_window_new (NULL, NULL);
+	gtk_widget_show (scrolledwindow);
+	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow),
+											  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow),
+													 GTK_SHADOW_IN);
+
+	gtk_container_add (GTK_CONTAINER (win), scrolledwindow);
 
 	vbox = gtk_vbox_new (FALSE, 5);
 	gtk_container_set_border_width (GTK_CONTAINER (vbox), 6);
-	gtk_container_add (GTK_CONTAINER (win), vbox);
+	gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW (scrolledwindow), vbox);
 
 	hbox = gtk_hbox_new (FALSE, 4);
 	gtk_container_add (GTK_CONTAINER (vbox), hbox);
@@ -1615,33 +1612,6 @@ setup_window_open (void)
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_end (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
 
-	/* prepare the button box */
-	hbbox = gtk_hbutton_box_new ();
-	gtk_box_set_spacing (GTK_BOX (hbbox), 4);
-	gtk_box_pack_end (GTK_BOX (hbox), hbbox, FALSE, FALSE, 0);
-
-	/* standard buttons */
-	/* GNOME doesn't like apply */
-#if 0
-	wid = gtk_button_new_from_stock (GTK_STOCK_APPLY);
-	g_signal_connect (G_OBJECT (wid), "clicked",
-							G_CALLBACK (setup_apply_cb), win);
-	gtk_box_pack_start (GTK_BOX (hbbox), wid, FALSE, FALSE, 0);
-#endif
-
-	cancel_button = wid = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
-	g_signal_connect (G_OBJECT (wid), "clicked",
-							G_CALLBACK (gtkutil_destroy), win);
-	gtk_box_pack_start (GTK_BOX (hbbox), wid, FALSE, FALSE, 0);
-
-	wid = gtk_button_new_from_stock (GTK_STOCK_OK);
-	g_signal_connect (G_OBJECT (wid), "clicked",
-							G_CALLBACK (setup_ok_cb), win);
-	gtk_box_pack_start (GTK_BOX (hbbox), wid, FALSE, FALSE, 0);
-
-	wid = gtk_hseparator_new ();
-	gtk_box_pack_end (GTK_BOX (vbox), wid, FALSE, FALSE, 0);
-
 	gtk_widget_show_all (win);
 
 	return win;
@@ -1660,10 +1630,11 @@ setup_open (void)
 
 	if (setup_window)
 	{
-		gtk_window_present (GTK_WINDOW (setup_window));
+		fe_message (_("Preferences editor is already open."), FE_MSG_ERROR);
 		return;
 	}
 
+
 	color_change = FALSE;
 	setup_window = setup_window_open ();
 
